from libcpp.vector cimport vector as libcpp_vector
from libcpp.list cimport list as libcpp_list
from DefaultParamHandler cimport *
from ProteinIdentification cimport *
from ConsensusMap cimport *
from FASTAFile cimport *
from String cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ProteinResolver.h>" namespace "OpenMS":
    cdef cppclass ProteinResolver(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        ProteinResolver() nogil except +
        ProteinResolver(ProteinResolver) nogil except +

        void resolveConsensus(ConsensusMap & consensus) nogil except +
        void resolveID(libcpp_vector[PeptideIdentification] & peptide_identifications) nogil except +
        void setProteinData(libcpp_vector[FASTAEntry] & protein_data) nogil except +
        libcpp_vector[ResolverResult] getResults() nogil except +

        void countTargetDecoy(libcpp_vector[ MSDGroup ] & msd_groups, ConsensusMap & consensus) nogil except +
        void countTargetDecoy(libcpp_vector[ MSDGroup ] & msd_groups, libcpp_vector[ PeptideIdentification ] & peptide_nodes) nogil except +
        void clearResult() nogil except +

        PeptideIdentification getPeptideIdentification(ConsensusMap & consensus, PeptideEntry * peptide) nogil except +
        PeptideHit getPeptideHit(ConsensusMap & consensus, PeptideEntry * peptide) nogil except +
        PeptideIdentification getPeptideIdentification(libcpp_vector[ PeptideIdentification ] & peptide_nodes, PeptideEntry * peptide) nogil except +
        PeptideHit getPeptideHit(libcpp_vector[ PeptideIdentification ] & peptide_nodes, PeptideEntry * peptide) nogil except +

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ProteinResolver.h>" namespace "OpenMS::ProteinResolver":
    
    cdef cppclass ISDGroup "OpenMS::ProteinResolver::ISDGroup":
        ISDGroup() nogil except +
        ISDGroup(ISDGroup) nogil except + #wrap-ignore

        # NAMESPACE # # POINTER # std::list[ ProteinEntry * ] proteins
        # NAMESPACE # # POINTER # std::list[ PeptideEntry * ] peptides
        Size index
        # libcpp_list[ size_t ] msd_groups # TODO no converter for List

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ProteinResolver.h>" namespace "OpenMS::ProteinResolver":
    
    cdef cppclass MSDGroup "OpenMS::ProteinResolver::MSDGroup":
        MSDGroup() nogil except +
        MSDGroup(MSDGroup) nogil except + #wrap-ignore

        # NAMESPACE # # POINTER # std::list[ ProteinEntry * ] proteins
        # NAMESPACE # # POINTER # std::list[ PeptideEntry * ] peptides
        Size index
        ISDGroup * isd_group
        Size number_of_decoy
        Size number_of_target
        Size number_of_target_plus_decoy
        float intensity

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ProteinResolver.h>" namespace "OpenMS::ProteinResolver":

    cdef cppclass ResolverResult "OpenMS::ProteinResolver::ResolverResult":
        ResolverResult() nogil except +
        ResolverResult(ResolverResult) nogil except +

        String identifier
        libcpp_vector[ISDGroup] * isds
        libcpp_vector[MSDGroup] * msds
        libcpp_vector[ProteinEntry] * protein_entries
        libcpp_vector[PeptideEntry] * peptide_entries
        # libcpp_vector[size_t] * reindexed_peptides
        # libcpp_vector[size_t] * reindexed_proteins
        ProteinResolverResult_Type input_type
        libcpp_vector[PeptideIdentification] * peptide_identification
        ConsensusMap * consensus_map

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ProteinResolver.h>" namespace "OpenMS::ProteinResolver":
    
    cdef cppclass PeptideEntry "OpenMS::ProteinResolver::PeptideEntry":
        PeptideEntry() nogil except +
        PeptideEntry(PeptideEntry) nogil except + #wrap-ignore

        # NAMESPACE # # POINTER # std::list[ ProteinEntry * ] proteins
        bool traversed
        String sequence
        Size peptide_identification
        Size peptide_hit
        Size index
        Size msd_group
        Size isd_group
        bool experimental
        float intensity
        String origin

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ProteinResolver.h>" namespace "OpenMS::ProteinResolver::ResolverResult":
    
    cdef enum ProteinResolverResult_Type "OpenMS::ProteinResolver::ResolverResult::type":
        PeptideIdent
        Consensus

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ProteinResolver.h>" namespace "OpenMS::ProteinResolver":
    
    cdef cppclass ProteinEntry "OpenMS::ProteinResolver::ProteinEntry":
        ProteinEntry() nogil except +
        ProteinEntry(ProteinEntry) nogil except + #wrap-ignore

        # NAMESPACE # # POINTER # std::list[ PeptideEntry * ] peptides
        bool traversed
        FASTAEntry * fasta_entry
        ProteinEntry_type protein_type
        double weight
        float coverage
        # NAMESPACE # # POINTER # std::list[ ProteinEntry * ] indis
        Size index
        Size msd_group
        Size isd_group
        Size number_of_experimental_peptides

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ProteinResolver.h>" namespace "OpenMS::ProteinResolver::ProteinEntry":

    cdef enum ProteinEntry_type "OpenMS::ProteinResolver::ProteinEntry::type":
        #wrap-attach:
        #    ProteinEntry

        primary
        secondary
        primary_indistinguishable
        secondary_indistinguishable




from Types cimport *
from ConsensusMap cimport *
from Peak2D cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ProteinInference.h>" namespace "OpenMS":
    
    cdef cppclass ProteinInference "OpenMS::ProteinInference":
        ProteinInference() nogil except +
        ProteinInference(ProteinInference) nogil except +
        void infer(ConsensusMap & consensus_map, UInt reference_map) nogil except +




import unittest
import os

import pyopenms

class TestMSNumpressCoder(unittest.TestCase):

    def setUp(self):
        self.testData = [ 
            100.0,
            200.0,
            300.00005,
            400.00010,
        ]

    def test_encodeNP_SLOF(self):
        """
          String out;

          MSNumpressCoder::NumpressConfig config;
          config.np_compression = MSNumpressCoder::SLOF;
          config.estimate_fixed_point = true; // critical

          bool zlib_compression = false;
          MSNumpressCoder().encodeNP(in, out, zlib_compression, config);

          TEST_EQUAL(out.size(), 24)
          TEST_EQUAL(out, "QMVagAAAAAAZxX3ivPP8/w==")

        """
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.SLOF
        config.estimate_fixed_point = True

        out_ = pyopenms.String()
        coder.encodeNP(self.testData, out_, False, config)
        out = out_.c_str()

        self.assertEqual( len(out),  24)
        self.assertEqual( out,  b"QMVagAAAAAAZxX3ivPP8/w==")

    def test_decodeNP_SLOF(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.SLOF
        config.estimate_fixed_point = True

        inData = b"QMVagAAAAAAZxX3ivPP8/w=="
        out = []
        coder.decodeNP(inData, out, False, config)

        self.assertEqual( len(out),  4)
        for a,b in zip(self.testData, out):
            self.assertAlmostEqual( a, b, places=2)

    def test_encodeNP_PIC(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.PIC
        config.estimate_fixed_point = True

        out_ = pyopenms.String()
        coder.encodeNP(self.testData, out_, False, config)
        out = out_.c_str()

        self.assertEqual( len(out),  12)
        self.assertEqual( out, b"ZGaMXCFQkQ==")

    def test_decodeNP_PIC(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.PIC
        config.estimate_fixed_point = True

        inData = b"ZGaMXCFQkQ=="
        out = []
        coder.decodeNP(inData, out, False, config)

        self.assertEqual( len(out),  4)
        for a,b in zip(self.testData, out):
            self.assertAlmostEqual( a, b, places=2)

    def test_encodeNP_LINEAR(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.LINEAR
        config.estimate_fixed_point = True

        out_ = pyopenms.String()
        coder.encodeNP(self.testData, out_, False, config)
        out = out_.c_str()

        self.assertEqual( len(out),  28)
        self.assertEqual( out,  b"QWR64UAAAADo//8/0P//f1kSgA==")

    def test_decodeNP_LINEAR(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.LINEAR
        config.estimate_fixed_point = True

        inData = b"QWR64UAAAADo//8/0P//f1kSgA=="
        out = []
        coder.decodeNP(inData, out, False, config)

        self.assertEqual( len(out),  4)
        for a,b in zip(self.testData, out):
            self.assertAlmostEqual( a, b, places=7)

class TestMSNumpressCoderRaw(unittest.TestCase):

    def setUp(self):
        self.testData = [ 
            100.0,
            200.0,
            300.00005,
            400.00010,
        ]

    def test_encodeNP_SLOF(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.SLOF
        config.estimate_fixed_point = True

        out_ = pyopenms.String()
        coder.encodeNPRaw(self.testData, out_, config)
        out = out_.c_str()

        self.assertEqual( len(out),  16)
        self.assertEqual( out,  b'@\xc5Z\x80\x00\x00\x00\x00\x19\xc5}\xe2\xbc\xf3\xfc\xff' )

    def test_decodeNP_SLOF(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.SLOF
        config.estimate_fixed_point = True

        inData = b'@\xc5Z\x80\x00\x00\x00\x00\x19\xc5}\xe2\xbc\xf3\xfc\xff'
        out = []
        coder.decodeNPRaw(inData, out, config)

        self.assertEqual( len(out),  4)
        for a,b in zip(self.testData, out):
            self.assertAlmostEqual( a, b, places=2)

    def test_encodeNP_PIC(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.PIC
        config.estimate_fixed_point = True

        out_ = pyopenms.String()
        coder.encodeNPRaw(self.testData, out_, config)
        out = out_.c_str()

        self.assertEqual( len(out),  7)
        self.assertEqual( out, b'df\x8c\\!P\x91')

    def test_decodeNP_PIC(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.PIC
        config.estimate_fixed_point = True

        inData = b'df\x8c\\!P\x91'
        out = []
        coder.decodeNPRaw(inData, out, config)

        self.assertEqual( len(out),  4)
        for a,b in zip(self.testData, out):
            self.assertAlmostEqual( a, b, places=2)

    def test_encodeNP_LINEAR(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.LINEAR
        config.estimate_fixed_point = True

        out_ = pyopenms.String()
        coder.encodeNPRaw(self.testData, out_, config)
        out = out_.c_str()

        self.assertEqual( len(out),  19)
        self.assertEqual( out,  b'Adz\xe1@\x00\x00\x00\xe8\xff\xff?\xd0\xff\xff\x7fY\x12\x80')

    def test_decodeNP_LINEAR(self):
        coder = pyopenms.MSNumpressCoder()
        config = pyopenms.NumpressConfig()
        config.np_compression = pyopenms.MSNumpressCoder.NumpressCompression.LINEAR
        config.estimate_fixed_point = True

        inData = b'Adz\xe1@\x00\x00\x00\xe8\xff\xff?\xd0\xff\xff\x7fY\x12\x80'
        out = []
        coder.decodeNPRaw(inData, out, config)

        self.assertEqual( len(out),  4)
        for a,b in zip(self.testData, out):
            self.assertAlmostEqual( a, b, places=7)

if __name__ == '__main__':
    unittest.main()



from MSSpectrum cimport *
from ExperimentalDesign cimport *
from FeatureMap cimport *
from ConsensusMap cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from Param cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *
from ProteinIdentification cimport *
# ctypedef libcpp_map<UInt64, double> SampleAbundances;

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/PeptideAndProteinQuant.h>" namespace "OpenMS":

    cdef cppclass PeptideAndProteinQuant(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        PeptideAndProteinQuant() nogil except +
        PeptideAndProteinQuant(PeptideAndProteinQuant) nogil except + #wrap-ignore

        void readQuantData(FeatureMap & map_in, ExperimentalDesign & ed) nogil except +
        void readQuantData(ConsensusMap & map_in, ExperimentalDesign & ed) nogil except +
        void readQuantData(libcpp_vector[ProteinIdentification] & proteins,
                           libcpp_vector[PeptideIdentification] & peptides,
                           ExperimentalDesign & ed) nogil except +

        void quantifyPeptides(libcpp_vector[PeptideIdentification] & peptides) nogil except +
        void quantifyProteins(ProteinIdentification & proteins) nogil except +

        PeptideAndProteinQuant_Statistics getStatistics() nogil except +

        # ctypedef libcpp_map<String, ProteinData] ProteinQuant
        # ctypedef libcpp_map<AASequence, PeptideData] PeptideQuant
        # PeptideQuant getPeptideResults() nogil except +
        # ProteinQuant getProteinResults() nogil except +

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/PeptideAndProteinQuant.h>" namespace "OpenMS::PeptideAndProteinQuant":

    cdef cppclass PeptideAndProteinQuant_Statistics "OpenMS::PeptideAndProteinQuant::Statistics":
      # number of samples
      Size n_samples

      # protein statistics
      Size quant_proteins
      Size too_few_peptides

      # peptide statistics
      Size quant_peptides
      Size total_peptides

      # feature statistics
      Size quant_features
      Size total_features
      Size blank_features
      Size ambig_features

      # constructor
      PeptideAndProteinQuant_Statistics()  nogil except +
      PeptideAndProteinQuant_Statistics(PeptideAndProteinQuant_Statistics) # wrap-ignore

    cdef cppclass PeptideAndProteinQuant_PeptideData "OpenMS::PeptideAndProteinQuant::PeptideData":

      # libcpp_map[Int, SampleAbundances] abundances
      # libcpp_map[unsigned long, double] total_abundances

      # protein accessions for this peptide
      libcpp_set[String] accessions

      # number of identifications
      Size psm_count

      # constructor
      PeptideAndProteinQuant_PeptideData()  nogil except +
      PeptideAndProteinQuant_PeptideData(PeptideAndProteinQuant_PeptideData) # wrap-ignore

    # Quantitative and associated data for a protein
    cdef cppclass PeptideAndProteinQuant_ProteinData "OpenMS::PeptideAndProteinQuant::ProteinData":

      # libcpp_map[String, SampleAbundances] abundances
 
      # compile issue 
      # libcpp_map[unsigned long, double] total_abundances

      # total number of identifications (of peptides mapping to this protein)
      Size psm_count

      # constructor
      PeptideAndProteinQuant_ProteinData()  nogil except +
      PeptideAndProteinQuant_ProteinData(PeptideAndProteinQuant_ProteinData) # wrap-ignore




from ProgressLogger cimport *
from DefaultParamHandler cimport *
from MSSpectrum cimport *
from Peak1D cimport *
from MSExperiment cimport *
from MzTab cimport *
from PeptideHit cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/ANALYSIS/ID/MetaboliteSpectralMatching.h>" namespace "OpenMS":

    cdef cppclass MetaboliteSpectralMatching(ProgressLogger, DefaultParamHandler):
        # wrap-inherits:
        #    ProgressLogger
        #    DefaultParamHandler

        MetaboliteSpectralMatching() nogil except +
        MetaboliteSpectralMatching(MetaboliteSpectralMatching) nogil except + 

        void run(MSExperiment & exp, MSExperiment & speclib, MzTab & mz_tab) nogil except +
        
    cdef cppclass SpectralMatch:

        SpectralMatch() nogil except +
        SpectralMatch(SpectralMatch) nogil except + 

        double getObservedPrecursorMass() nogil except +
        void setObservedPrecursorMass(double)nogil except +

        double getObservedPrecursorRT() nogil except +
        void setObservedPrecursorRT(double)nogil except +

        double getFoundPrecursorMass() nogil except +
        void setFoundPrecursorMass(double)nogil except +

        Int getFoundPrecursorCharge() nogil except +
        void setFoundPrecursorCharge(Int)nogil except +

        double getMatchingScore() nogil except +
        void setMatchingScore(double)nogil except +

        Size getObservedSpectrumIndex() nogil except +
        void setObservedSpectrumIndex(Size)nogil except +

        Size getMatchingSpectrumIndex() nogil except +
        void setMatchingSpectrumIndex(Size)nogil except +

        String getPrimaryIdentifier() nogil except +
        void setPrimaryIdentifier(String)nogil except +

        String getSecondaryIdentifier() nogil except +
        void setSecondaryIdentifier(String)nogil except +

        String getCommonName() nogil except +
        void setCommonName(String)nogil except +

        String getSumFormula() nogil except +
        void setSumFormula(String)nogil except +

        String getInchiString() nogil except +
        void setInchiString(String)nogil except +

        String getSMILESString() nogil except +
        void setSMILESString(String)nogil except +

        String getPrecursorAdduct() nogil except +
        void setPrecursorAdduct(String)nogil except +

# COMMENT: wrap static methods
cdef extern from "<OpenMS/ANALYSIS/ID/MetaboliteSpectralMatching.h>" namespace"OpenMS::MetaboliteSpectralMatching":
        
        # static members        
        # double computeHyperScore(double fragment_mass_error,
        #                          bool fragment_mass_tolerance_unit_ppm,
        #                          MSSpectrum exp_spectrum,
        #                          MSSpectrum db_spectrum) nogil except +  # wrap-attach:MetaboliteSpectralMatching
        # 
        # double computeHyperScore(double fragment_mass_error,
        #                          bool fragment_mass_tolerance_unit_ppm,
        #                          MSSpectrum exp_spectrum,
        #                          MSSpectrum db_spectrum,
        #                          double mz_lower_bound) nogil except + # wrap-attach:MetaboliteSpectralMatching
        # 
        # double computeHyperScore(double fragment_mass_error,
        #                          bool fragment_mass_tolerance_unit_ppm,
        #                          MSSpectrum exp_spectrum,
        #                          MSSpectrum db_spectrum,
        #                          libcpp_vector[PeptideHit_PeakAnnotation]& annotations) nogil except + 
        #     
        double computeHyperScore(double fragment_mass_error,
                                 bool fragment_mass_tolerance_unit_ppm,
                                 MSSpectrum exp_spectrum,
                                 MSSpectrum db_spectrum,
                                 libcpp_vector[PeptideHit_PeakAnnotation]& annotations,
                                 double mz_lower_bound) nogil except + # wrap-attach:MetaboliteSpectralMatching
        



from libcpp.vector cimport vector as libcpp_vector
from libcpp.string cimport string as libcpp_string
from Peak1D cimport *
from String cimport *
from RangeManager cimport *
from DataArrays cimport *
from SpectrumSettings cimport *

# this class has addons, see the ./addons folder (../addons/MSSpectrum.pyx)

cdef extern from "<OpenMS/KERNEL/MSSpectrum.h>" namespace "OpenMS":

    cdef cppclass MSSpectrum(SpectrumSettings, RangeManager1):
        # wrap-inherits:
        #  SpectrumSettings
        #  RangeManager1
        #
        # wrap-doc:
        #   The representation of a 1D spectrum.
        #   Raw data access is proved by `get_peaks` and `set_peaks`, which yields numpy arrays
        #   Iterations yields access to underlying peak objects but is slower
        #   Extra data arrays can be accessed through getFloatDataArrays / getIntegerDataArrays / getStringDataArrays
        #   See help(SpectrumSettings) for information about meta-information
        #   -----
        #   Usage:
        #     ms_level = spectrum.getMSLevel()
        #     rt = spectrum.getRT()
        #     mz, intensities = spectrum.get_peaks()
        #   -----

        MSSpectrum() nogil except +
        MSSpectrum(MSSpectrum &) nogil except +
        double getRT() nogil except +
        void setRT(double) nogil except +
        double getDriftTime() nogil except +
        void setDriftTime(double) nogil except +
        unsigned int getMSLevel() nogil except +
        void setMSLevel(unsigned int) nogil except +

        String getName() nogil except +
        void setName(String) nogil except +

        Size size() nogil except +
        void reserve(size_t n) nogil except + 

        Peak1D& operator[](int) nogil except + # wrap-upper-limit:size()

        void updateRanges() nogil except +
        void clear(bool clear_meta_data) nogil except + #wrap-doc:Clears all data (and meta data if clear_meta_data is true)
        void push_back(Peak1D)  nogil except + #wrap-doc:Append a peak

        bool isSorted() nogil except +

        int findNearest(double) nogil except+
        int findNearest(double, double) nogil except+
        int findNearest(double, double, double) nogil except+
        int findHighestInWindow(double, double, double) nogil except+

        MSSpectrum select(libcpp_vector[ size_t ] & indices) nogil except +

        void assign(libcpp_vector[Peak1D].iterator, libcpp_vector[Peak1D].iterator) nogil except + # wrap-ignore
        libcpp_vector[Peak1D].iterator begin() nogil except +  # wrap-iter-begin:__iter__(Peak1D)
        libcpp_vector[Peak1D].iterator end()   nogil except +  # wrap-iter-end:__iter__(Peak1D)

        double getTIC() nogil except +

        bool operator==(MSSpectrum) nogil except +
        bool operator!=(MSSpectrum) nogil except +

        void sortByIntensity(bool reverse) nogil except +
        void sortByPosition() nogil except +

        libcpp_vector[FloatDataArray] getFloatDataArrays() nogil except +
        libcpp_vector[IntegerDataArray] getIntegerDataArrays() nogil except +
        libcpp_vector[StringDataArray] getStringDataArrays() nogil except +

        void setFloatDataArrays(libcpp_vector[FloatDataArray] fda) nogil except +
        void setIntegerDataArrays(libcpp_vector[IntegerDataArray] ida) nogil except +
        void setStringDataArrays(libcpp_vector[StringDataArray] sda) nogil except +








    def getSequences(self, String database_filename , dict ac_position_map, list sequences, list found, dict not_found):
        # void getSequences(String &database_filename,
        # libcpp_map[ String, Size ] &ac_position_map,
        # libcpp_vector[ String ] &sequences,
        # libcpp_vector[ libcpp_pair[ String, Size ] ] &found,
        # libcpp_map[ String, Size ] &not_found) nogil except +
        assert isinstance(database_filename, String), 'arg database_filename wrong type'
        assert isinstance(sequences, list) and all(isinstance(i, bytes) for i in sequences), 'arg sequences wrong type'
    
        cdef libcpp_vector[_String] * v1 = new libcpp_vector[_String]()
        cdef bytes item1
        for item1 in sequences:
           v1.push_back(_String(<char *>item1))

        cdef libcpp_map[_String, size_t ] c_dict_ac_position
        for k,v in ac_position_map.iteritems():
            c_dict_ac_position[ _String(<char *>k) ] = v

        cdef libcpp_map[_String, size_t ] c_dict_not_found
        for k,v in not_found.iteritems():
            c_dict_not_found[ _String(<char *>k) ] = v

        cdef libcpp_vector[ libcpp_pair[_String, size_t] ] * v4 = new libcpp_vector[ libcpp_pair[_String, size_t] ]()
        cdef list item4
        cdef libcpp_pair[_String, size_t] * aPair
        for item4 in found:

           aPair = new libcpp_pair[ _String, size_t] ( _String(<char *>item4[0]), item4[1])
           v4.push_back( deref(aPair) )
           del aPair
 
        #
        # Call C++
        #
        self.inst.get().getSequences(deref(database_filename.inst.get()), c_dict_ac_position, deref(v1), deref(v4), c_dict_not_found)
        #
        # Get arguments back from C++
        #

        # libcpp_map[ String, Size ] &ac_position_map,
        replace_2 = dict()
        cdef libcpp_map[_String, size_t].iterator it_dict2 = c_dict_ac_position.begin()
        while it_dict2 != c_dict_ac_position.end():
            replace_2[ <libcpp_string>deref(it_dict2).first ] = deref(it_dict2).second
            inc(it_dict2)
        ac_position_map.update(replace_2)

        # libcpp_vector[ String ] &sequences,
        cdef replace = []
        cdef libcpp_vector[_String].iterator it = v1.begin()
        while it != v1.end():
           replace.append(<char*>deref(it).c_str())
           inc(it)
        sequences[:] = replace

        # libcpp_vector[ libcpp_pair[ String, Size ] ] &found,
        replace_4 = list()
        cdef libcpp_vector[ libcpp_pair[_String, size_t] ].iterator it_list4 = deref(v4).begin()
        cdef libcpp_pair[_String, size_t] anotherPair
        while it_list4 != deref(v4).end():
            # get the pair back
            anotherPair = deref(it_list4)
            replace_4.append( [<libcpp_string>anotherPair.first, anotherPair.second] )
            inc(it_list4)
        found[:] = replace_4

        # libcpp_map[ String, Size ] &not_found
        replace_5 = dict()
        cdef libcpp_map[_String, size_t].iterator it_dict5 = c_dict_not_found.begin()
        while it_dict5 != c_dict_not_found.end():
            replace_2[ <libcpp_string>deref(it_dict5).first ] = deref(it_dict5).second
            inc(it_dict5)
        not_found.update(replace_2)


        del v1
        del v4




import argparse
import pyopenms as pms
from common import addDataProcessing, writeParamsIfRequested, updateDefaults
from  collections import Counter


def link(in_files, out_file, keep_subelements, params):

    in_types = set(pms.FileHandler.getType(in_) for in_ in in_files)

    if in_types == set((pms.Type.CONSENSUSXML,)):
        link_features = False
    elif in_types == set((pms.Type.FEATUREXML,)):
        link_features = True
    else:
        raise Exception("different kinds of input files")

    algorithm_parameters = params.copy("algorithm:", True)
    algorithm = pms.FeatureGroupingAlgorithmQT()
    algorithm.setParameters(algorithm_parameters)

    out_map = pms.ConsensusMap()
    fds = out_map.getColumnHeaders()
    if link_features:
        f = pms.FeatureXMLFile()
        maps = []
        for i, in_file in enumerate(in_files):
            map_ = pms.FeatureMap()
            f.load(in_file, map_)

            # set filedescriptions
            fd = fds.get(i, pms.ColumnHeader())
            fd.filename = in_file
            fd.size = map_.size()
            fd.unique_id = map_.getUniqueId()
            fds[i] = fd
            maps.append(map_)
        out_map.setColumnHeaders(fds)
        algorithm.group(maps, out_map)
    else:
        f = pms.ConsensusXMLFile()
        maps = []
        for i, in_file in enumerate(in_files):
            map_ = pms.ConsensusMap()
            f.load(in_file, map_)
            maps.append(map_)
        algorithm.group(maps, out_map)

        if not keep_subelements:
            for i in range(len(in_files)):
                # set filedescriptions
                fd = fds.get(i, pms.ColumnHeader())
                fd.filename = in_files[i]
                fd.size = maps[i].size()
                fd.unique_id = maps[i].getUniqueId()
                fds[i] = fd
            out_map.setColumnHeaders(fds)
        else:
            algorithm.transferSubelements(maps, out_map)

    out_map.setUniqueIds()
    addDataProcessing(out_map, params, pms.ProcessingAction.FEATURE_GROUPING)

    pms.ConsensusXMLFile().store(out_file, out_map)

    sizes = []
    for feat in out_map:
        sizes.append(feat.size())

    c = Counter(sizes)
    print "Number of consensus features:"
    for size, count in c.most_common():
        print "   of size %2d : %6d" % (size, count)
    print "        total : %6d" % out_map.size()


def main():

    parser = argparse.ArgumentParser(description="FeatureLinkerUnlabeledQT")
    parser.add_argument("-in",
                        action="append",
                        type=str,
                        dest="in_",
                        metavar="input_files",
                        )

    parser.add_argument("-out",
                        action="store",
                        type=str,
                        metavar="output_file",
                        )

    parser.add_argument("-ini",
                        action="store",
                        type=str,
                        metavar="ini_file",
                        )

    parser.add_argument("-dict_ini",
                        action="store",
                        type=str,
                        metavar="python_dict_ini_file",
                        )

    parser.add_argument("-write_ini",
                        action="store",
                        type=str,
                        metavar="ini_file",
                        )

    parser.add_argument("-write_dict_ini",
                        action="store",
                        type=str,
                        metavar="python_dict_ini_file",
                        )

    parser.add_argument("-keep_subelements",
                        action="store_true",
                        )


    args = parser.parse_args()


    def collect(args):
        return [f.strip() for arg in args or [] for f in arg.split(",")]

    in_files = collect(args.in_)

    run_mode = (in_files and args.out) \
                and (args.ini is not None or args.dict_ini is not None)

    write_mode = args.write_ini is not None or args.write_dict_ini is not None
    ok = run_mode or write_mode
    if not ok:
        parser.error("either specify -in, -out and -(dict)ini for running "
                     "the feature linker\nor -write(dict)ini for creating std "
                     "ini file")

    defaults = pms.FeatureGroupingAlgorithmQT().getParameters()
    write_requested = writeParamsIfRequested(args, defaults)

    if not write_requested:
        updateDefaults(args, defaults)


        link(in_files, args.out, args.keep_subelements, defaults)



if __name__ == "__main__":
    main()



from libcpp cimport bool
from Types cimport *
from String cimport *
from DataValue cimport *

cdef extern from "<OpenMS/METADATA/CVTerm.h>" namespace "OpenMS":

    cdef cppclass CVTerm:
         CVTerm()   nogil except +
         CVTerm(CVTerm)   nogil except +

         bool operator==(CVTerm)   nogil except +

         void setAccession(String accession) nogil except +
         String getAccession() nogil except +

         void setName(String name) nogil except +
         String getName() nogil except +

         void setCVIdentifierRef(String cv_id_ref) nogil except +
         String getCVIdentifierRef() nogil except +

         DataValue getValue()   nogil except +
         void setValue(DataValue value) nogil except +

         void setUnit(Unit & unit) nogil except +
         Unit  getUnit() nogil except +
         bool hasValue() nogil except +
         bool hasUnit() nogil except +

cdef extern from "<OpenMS/METADATA/CVTerm.h>" namespace "OpenMS::CVTerm":
    
    cdef cppclass Unit "OpenMS::CVTerm::Unit":
        Unit() nogil except +
        Unit(Unit) nogil except +
        String accession
        String name
        String cv_ref
        Unit(const String & p_accession, const String & p_name, const String & p_cv_ref) nogil except +
        bool operator==(Unit & rhs) nogil except +
        bool operator!=(Unit & rhs) nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from Map cimport *
from String cimport *
from ResidueModification cimport *
#from libcpp.memory cimport unique_ptr as libcpp_unique_ptr

# see ../addons/CrossLinksDB.pyx
cdef extern from "<OpenMS/CHEMISTRY/CrossLinksDB.h>" namespace "OpenMS":
    
    cdef cppclass CrossLinksDB:
        # wrap-manual-memory:
        #     cdef AutowrapPtrHolder[_CrossLinksDB] inst

        CrossLinksDB(CrossLinksDB) nogil except + #wrap-ignore

        Size getNumberOfModifications() nogil except +

        void searchModifications(libcpp_set[ const ResidueModification * ] & mods,
                                 const String& mod_name,
                                 const String& residue,
                                 TermSpecificity term_spec) nogil except +

        const ResidueModification * getModification(Size index) nogil except +

        const ResidueModification * getModification(const String & mod_name) nogil except +

        const ResidueModification * getModification(const String & mod_name,
                                            const String & residue,
                                            TermSpecificity term_spec) nogil except +

        bool has(String modification) nogil except +

        #void addModification(libcpp_unique_ptr[ResidueModification] new_mod) nogil except +

        Size findModificationIndex(const String & mod_name) nogil except +

        void searchModificationsByDiffMonoMass(libcpp_vector[ String ] & mods, double mass, double max_error,
                                               const String & residue, TermSpecificity term_spec) nogil except +

        const ResidueModification* getBestModificationByDiffMonoMass(double mass, double max_error,
                                                              const String residue, TermSpecificity term_spec) nogil except +
        void getAllSearchModifications(libcpp_vector[ String ] & modifications) nogil except +

        void readFromOBOFile(const String & filename) nogil except +

        bool isInstantiated() nogil except +

## wrap static methods
cdef extern from "<OpenMS/CHEMISTRY/CrossLinksDB.h>" namespace "OpenMS::CrossLinksDB":
    
    CrossLinksDB* getInstance() nogil except + # wrap-ignore




from libc.stddef cimport *
from ctime cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from libcpp.pair cimport pair as libcpp_pair
from libcpp.map cimport map as libcpp_map
from libcpp.set cimport set as libcpp_set
from libcpp.string cimport string as libcpp_string
from smart_ptr cimport shared_ptr

# include macros
cdef extern from "<OpenMS/config.h>":
    pass

cdef extern from "<OpenMS/CONCEPT/Types.h>" namespace "OpenMS":

    ctypedef signed int Int32     "OPENMS_INT32_TYPE"
    ctypedef signed long Int64    "OPENMS_INT64_TYPE"
    ctypedef unsigned int UInt32  "OPENMS_UINT32_TYPE"
    ctypedef unsigned long UInt64 "OPENMS_UINT64_TYPE"
    ctypedef time_t   Time
    ctypedef unsigned int UInt
    ctypedef int      Int
    ctypedef unsigned long   UID "OPENMS_UINT64_TYPE"
    ctypedef size_t    Size
    ctypedef ptrdiff_t SignedSize



from Types cimport *
from ChromatogramSettings cimport *
from ChromatogramPeak cimport *
from String cimport *
from RangeManager cimport *
from MetaInfoDescription cimport *
from DataArrays cimport *

# this class has addons, see the ./addons folder

cdef extern from "<OpenMS/KERNEL/MSChromatogram.h>" namespace "OpenMS":

    cdef cppclass MSChromatogram (ChromatogramSettings, RangeManager1):
        # wrap-inherits:
        #  ChromatogramSettings
        #  RangeManager1
        #
        # wrap-doc:
        #   The representation of a chromatogram.
        #   Raw data access is proved by `get_peaks` and `set_peaks`, which yields numpy arrays
        #   Iterations yields access to underlying peak objects but is slower
        #   Extra data arrays can be accessed through getFloatDataArrays / getIntegerDataArrays / getStringDataArrays
        #   See help(ChromatogramSettings) for information about meta-information
        #   -----
        #   Usage:
        #     precursor = chromatogram.getPrecursor()
        #     product = chromatogram.getProduct()
        #     rt, intensities = chromatogram.get_peaks()
        #   -----

        MSChromatogram() nogil except +
        MSChromatogram(MSChromatogram &) nogil except +
        double getMZ() nogil except + #wrap-doc:returns the mz of the product entry, makes sense especially for MRM scans
        # void   setMZ(double) nogil except +

        String getName() nogil except +
        void setName(String) nogil except +

        Size size() nogil except +
        void reserve(size_t n) nogil except + 

        ChromatogramPeak & operator[](int) nogil except +

        void updateRanges() nogil except +
        void clear(int) nogil except +
        void push_back(ChromatogramPeak)  nogil except + #wrap-doc:Append a peak

        bool isSorted() nogil except +

        void sortByIntensity(bool reverse) nogil except +
        void sortByPosition() nogil except +

        int findNearest(double) nogil except+

        void assign(libcpp_vector[ChromatogramPeak].iterator, libcpp_vector[ChromatogramPeak].iterator) nogil except + # wrap-ignore
        libcpp_vector[ChromatogramPeak].iterator begin() nogil except +  # wrap-iter-begin:__iter__(ChromatogramPeak)
        libcpp_vector[ChromatogramPeak].iterator end()   nogil except +  # wrap-iter-end:__iter__(ChromatogramPeak)

        libcpp_vector[FloatDataArray] getFloatDataArrays() nogil except +
        libcpp_vector[IntegerDataArray] getIntegerDataArrays() nogil except +
        libcpp_vector[StringDataArray] getStringDataArrays() nogil except +

        void setFloatDataArrays(libcpp_vector[FloatDataArray] fda) nogil except +
        void setIntegerDataArrays(libcpp_vector[IntegerDataArray] ida) nogil except +
        void setStringDataArrays(libcpp_vector[StringDataArray] sda) nogil except +




# import pyopenms

def main():
    import optparse
    # in: file
    in_formats = "mzData,mzXML,mzML,DTA,DTA2D,mgf,featureXML,consensusXML,"\
                 "ms2,fid,tsv,peplist,kroenik,edta".split()
    # out: file
    out_formats = "mzData,mzXML,mzML,DTA2D,mgf,featureXML,"\
                  "consensusXML".split(",")

if __name__ == "__main__":
    main()



from Types cimport *
from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from PeakMarker cimport *

cdef extern from "<OpenMS/FILTERING/TRANSFORMERS/ComplementMarker.h>" namespace "OpenMS":
    
    cdef cppclass ComplementMarker(PeakMarker) :
        # wrap-inherits:
        #  PeakMarker
        ComplementMarker() nogil except +
        ComplementMarker(ComplementMarker) nogil except +
        void apply(libcpp_map[ double, bool ] & , MSSpectrum & ) nogil except +
        PeakMarker * create() nogil except + # wrap-ignore
        # String getProductName() nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from libcpp.set cimport set as libcpp_set
from libcpp.vector cimport vector as libcpp_vector
from PeptideIdentification cimport *
from LPWrapper cimport *

cdef extern from "<OpenMS/ANALYSIS/TARGETED/PSProteinInference.h>" namespace "OpenMS":
    
    cdef cppclass PSProteinInference "OpenMS::PSProteinInference":
        PSProteinInference() nogil except +
        PSProteinInference(PSProteinInference) nogil except + #wrap-ignore
        Size findMinimalProteinList(libcpp_vector[ PeptideIdentification ] & peptide_ids) nogil except +
        void calculateProteinProbabilities(libcpp_vector[ PeptideIdentification ] & ids) nogil except +
        double getProteinProbability(const String & acc) nogil except +
        bool isProteinInMinimalList(const String & acc) nogil except +
        Int getNumberOfProtIds(double protein_id_threshold) nogil except +
        # TODO nested STL
        # Int getNumberOfProtIdsPeptideRule(Int min_peptides, libcpp_map[ String, libcpp_set[ String ] ] & prot_id_counter) nogil except +
        SOLVER getSolver() nogil except +




import unittest
import os

import pyopenms

class TestGaussFilter(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test2.mzML").encode()
        self.exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(self.filename, self.exp)

    def test_init(self):
        thisfilter = pyopenms.GaussFilter();

    def test_run(self):
        thisfilter = pyopenms.GaussFilter();
        old_firstspec = self.exp[0]
        thisfilter.filterExperiment(self.exp)

        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

        # MZ should not change, Intensity should
        self.assertEqual(old_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(old_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

class TestSavitzkyGolayFilter(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test2.mzML").encode()
        self.exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(self.filename, self.exp)

    def test_init(self):
        thisfilter = pyopenms.SavitzkyGolayFilter();

    def test_run(self):
        thisfilter = pyopenms.SavitzkyGolayFilter();
        old_firstspec = self.exp[0]
        thisfilter.filterExperiment(self.exp)

        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

        # MZ should not change, Intensity should
        self.assertEqual(old_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(old_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

class TestLowessSmoothing(unittest.TestCase):

    def setUp(self):
        pass

    def test_init(self):
        thisfilter = pyopenms.LowessSmoothing();

    def test_init(self):
        thisfilter = pyopenms.LowessSmoothing();
        x = [1.0,2.0,3.0,4.0]
        y = [10.0,11.0,12.0,13.0]
        y_smoothed = [0.0]
        thisfilter.smoothData(x,y,y_smoothed)

        self.assertNotEqual( len(y_smoothed), 0)

        # The smoothed data should be different from the input data
        self.assertNotEqual( y_smoothed, y)
        self.assertNotEqual( y_smoothed[0], y[0])

if __name__ == '__main__':
    unittest.main()



from Types cimport *
from String cimport *
from ProgressLogger cimport *
from TargetedExperiment cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMDecoy.h>" namespace "OpenMS":

    cdef cppclass MRMDecoy(ProgressLogger):
        # wrap-inherits:
        #    ProgressLogger

        MRMDecoy()                       nogil except +
        MRMDecoy(MRMDecoy)               nogil except + # wrap-ignore

        void generateDecoys(TargetedExperiment& exp,
                            TargetedExperiment& dec,
                            String method,
                            double aim_decoy_fraction,
                            bool switchKR,
                            String decoy_tag,
                            int max_attempts,
                            double identity_threshold,
                            double precursor_mz_shift,
                            double product_mz_shift,
                            double product_mz_threshold,
                            libcpp_vector[String] fragment_types,
                            libcpp_vector[size_t] fragment_charges,
                            bool enable_specific_losses,
                            bool enable_unspecific_losses,
                            int round_decPow) nogil except +

        libcpp_vector[size_t] findFixedResidues(const String & sequence,
                                                bool keepN,
                                                bool keepC,
                                                const String & keep_const_pattern) nogil except +




#!/usr/bin/env python
# -*- coding: utf-8  -*-
"""
--------------------------------------------------------------------------
                  OpenMS -- Open-Source Mass Spectrometry
--------------------------------------------------------------------------
Copyright The OpenMS Team -- Eberhard Karls University Tuebingen,
ETH Zurich, and Freie Universitaet Berlin 2002-2020.

This software is released under a three-clause BSD license:
 * Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
 * Neither the name of any author or any participating institution
   may be used to endorse or promote products derived from this software
   without specific prior written permission.
For a full list of authors, refer to the file AUTHORS.
--------------------------------------------------------------------------
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--------------------------------------------------------------------------
$Maintainer: Hannes Roest $
$Authors: Hannes Roest $
--------------------------------------------------------------------------
"""
from __future__ import division
from __future__ import print_function

# Create simulated ion mobility scans for testing

from pyopenms import *

exp = MSExperiment()
# print(dir(exp))

NR_RT_SAMPLES = 50
NR_IM_BINS = 300
NR_PEAKS = 5

# Create MS1 spectra
for rt_idx in range(NR_RT_SAMPLES):
    sp = MSSpectrum()
    sp.setRT(rt_idx)
    sp.setMSLevel(1)

    base_int = 100 - abs(20 - rt_idx)*(100)/25.0  # shift the precursor slightly
    base_int = max(5, base_int)

    fda = pyopenms.FloatDataArray()
    fda.setName("Ion Mobility")
    fda.resize(100)

    for i in range(100):
        p = Peak1D()
        p.setMZ(100+i)
        p.setIntensity(base_int+i)
        sp.push_back(p)
        fda[i] = 10

    for i in range(10):
        p = Peak1D()
        p.setMZ(412.502)
        p.setIntensity(base_int+ (i-5))
        sp.push_back(p)
        fda.push_back( 99 + (i-5) )

    for i in range(10):
        p = Peak1D()
        p.setMZ(417.502)
        p.setIntensity(base_int+ (i-5))
        sp.push_back(p)
        fda.push_back( 152 + (i-5) )

    sp.setFloatDataArrays([fda])
    exp.addSpectrum(sp)

# Create MS2 spectra
for rt_idx in range(NR_RT_SAMPLES):
    # Base intensity is a single peak at 25 RT with 100 *i intensity spread across ion mobility scans
    # Second base intensity is a single peak at 10 RT with 100 *i intensity spread across ion mobility scans
    base_int = 100 - abs(25 - rt_idx)*(100)/25.0
    base_int_second = 100 - abs(10 - rt_idx)*(100)/40.0
    print("base int", base_int, abs(25 - rt_idx)*(100)/25.0  )

    allmz = []
    allint = []
    allim = []

    for im_idx in range(NR_IM_BINS):
        sp = MSSpectrum()
        p = pyopenms.Precursor()
        p.setIsolationWindowLowerOffset(12)
        p.setIsolationWindowUpperOffset(12)
        target_mz = im_idx * 2.5 + 400
        p.setMZ(target_mz)
        sp.setPrecursors([p])

        sp.setRT(rt_idx+0.2)
        sp.setMSLevel(2)

        # peaks of a precursor at 412.5 m/z : 100, 101, 102, .. 100 + NR_PEAKS
        # and ion mobility 100
        for i in range(NR_PEAKS):
            if im_idx > 90 and im_idx < 90 + 20:
                apex_dist = abs( 100 - im_idx)
                p = Peak1D()
                p.setMZ(100+i)
                p.setIntensity(base_int * (i + 1) - base_int * (i + 1) * apex_dist / 10.0)
                allmz.append(p.getMZ())
                allint.append(p.getIntensity())
                allim.append( im_idx / 500.0)

        # peaks of a precursor at 417.5 m/z : 100, 101, 102, .. 100 + NR_PEAKS
        # and ion mobility 150
        for i in range(NR_PEAKS):
            if im_idx > 130 and im_idx < 130 + 40:
                apex_dist = abs( 150 - im_idx)
                p = Peak1D()
                p.setMZ(100+i)
                p.setIntensity(base_int * (i + 1) - base_int * (i + 1) * apex_dist / 20.0)
                allmz.append(p.getMZ())
                allint.append(p.getIntensity())
                allim.append( im_idx / 500.0)

    mz = allmz
    intens = allint
    ims = allim

    # print(mz, intens)

    fda = pyopenms.FloatDataArray()
    fda.setName("Ion Mobility")
    fda.resize(len(mz))
    for k,val in enumerate(ims):
        fda[k] = val

    sframe = pyopenms.MSSpectrum()
    sframe.setMSLevel(2)
    sframe.setRT(rt_idx+0.2)
    sframe.setFloatDataArrays([fda])
    p = pyopenms.Precursor()
    if True:
        center = 412.5
        width = 25
        p.setMZ(center)
        p.setIsolationWindowUpperOffset(width / 2.0)
        p.setIsolationWindowLowerOffset(width / 2.0)
    sframe.setPrecursors([p])
    sframe.set_peaks( (mz, intens) )
    sframe.sortByPosition()
    exp.addSpectrum(sframe)


f = MzMLFile()
pf = f.getOptions()
pf.setCompression(True)
f.setOptions(pf)
exp.sortSpectra()
f.store('output.mzML', exp)






    def getModifications(self):
        cdef libcpp_map[_String, libcpp_vector[_String] ] c_res 
        c_res = self.inst.get().getModifications()
        pass 
        # 
        ## Get the data back from C++
        #
        replace = dict()
        # Make sure each C++ and each Cython objects you use are declared here
        # (only pure Python objects need no declaration).
        cdef libcpp_map[_String, libcpp_vector[_String] ].iterator it_outer = c_res.begin()
        cdef libcpp_vector[_String] c_inner_vec
        cdef libcpp_vector[_String].iterator it_inner
        cdef _String myString
        cdef String py_String
        while it_outer != c_res.end():
            
            inner_vec = []
            c_inner_vec = deref(it_outer).second
            it_inner = c_inner_vec.begin()
            while it_inner != c_inner_vec.end():

                py_String = String.__new__(String)   
                py_String.inst = shared_ptr[_String](new _String(deref(it_inner)))
                inner_vec.append(py_String)
                inc(it_inner)

            replace[ <libcpp_string>deref(it_outer).first ] = inner_vec
            inc(it_outer)

        return replace






from String cimport *
from Types cimport *

cdef extern from "<OpenMS/METADATA/Software.h>" namespace "OpenMS":

    cdef cppclass Software:

        Software()   nogil except +
        Software(Software) nogil except + # wrap-ignore

        String getName() nogil except +
        String getVersion() nogil except +

        void setName(String) nogil except +
        void setVersion(String) nogil except +






from Types cimport *
from libcpp cimport bool
# from SeqanIncludeWrapper cimport *
from String cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/AhoCorasickAmbiguous.h>" namespace "OpenMS":
    
    cdef cppclass AhoCorasickAmbiguous "OpenMS::AhoCorasickAmbiguous":
        AhoCorasickAmbiguous() nogil except +
        AhoCorasickAmbiguous(AhoCorasickAmbiguous) nogil except + #wrap-ignore
        # void initPattern(const PeptideDB & pep_db, const int aaa_max, const int mm_max, FuzzyACPattern & pattern) nogil except +
        AhoCorasickAmbiguous(const String & protein_sequence) nogil except +
        void setProtein(const String & protein_sequence) nogil except +
        # bool findNext(const FuzzyACPattern & pattern) nogil except +
        Size getHitDBIndex() nogil except +
        Int getHitProteinPosition() nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from CVTermList cimport *
from TargetedExperimentHelper cimport *

cdef extern from "<OpenMS/ANALYSIS/TARGETED/IncludeExcludeTarget.h>" namespace "OpenMS":
    
    cdef cppclass IncludeExcludeTarget :
        IncludeExcludeTarget() nogil except +
        IncludeExcludeTarget(IncludeExcludeTarget) nogil except +
        void setName(const String & name) nogil except +
        String  getName() nogil except +
        void setPeptideRef(const String & peptide_ref) nogil except +
        String  getPeptideRef() nogil except +
        void setCompoundRef(const String & compound_ref) nogil except +
        String  getCompoundRef() nogil except +
        void setPrecursorMZ(double mz) nogil except +
        double getPrecursorMZ() nogil except +
        void setPrecursorCVTermList(CVTermList & list_) nogil except +
        void addPrecursorCVTerm(CVTerm & cv_term) nogil except +
        CVTermList  getPrecursorCVTermList() nogil except +
        void setProductMZ(double mz) nogil except +
        double getProductMZ() nogil except +
        void setProductCVTermList(CVTermList & list_) nogil except +
        void addProductCVTerm(CVTerm & cv_term) nogil except +
        CVTermList  getProductCVTermList() nogil except +
        void setInterpretations(libcpp_vector[ CVTermList ] & interpretations) nogil except +
        libcpp_vector[ CVTermList ]  getInterpretations() nogil except +
        void addInterpretation(CVTermList & interpretation) nogil except +
        void setConfigurations(libcpp_vector[ Configuration ] & configuration) nogil except +
        libcpp_vector[ Configuration ]  getConfigurations() nogil except +
        void addConfiguration(Configuration & configuration) nogil except +
        void setPrediction(CVTermList & prediction) nogil except +
        void addPredictionTerm(CVTerm & prediction) nogil except +
        CVTermList  getPrediction() nogil except +
        void setRetentionTime(RetentionTime rt) nogil except +
        RetentionTime  getRetentionTime() nogil except +
        bool operator==(IncludeExcludeTarget & rhs) nogil except +
        bool operator!=(IncludeExcludeTarget & rhs) nogil except +

        # from CVTermsList
        void setCVTerms(libcpp_vector[CVTerm] & terms)  nogil except +
        void replaceCVTerm(CVTerm & term)               nogil except +

        void replaceCVTerms(libcpp_vector[CVTerm] cv_terms,
                            String accession
                           ) nogil except +

        void replaceCVTerms(Map[String, libcpp_vector[CVTerm] ] cv_term_map
                           ) nogil except +

        Map[String, libcpp_vector[CVTerm] ] getCVTerms() nogil except +
        void addCVTerm(CVTerm & term)                   nogil except +

        bool hasCVTerm(String accession)  nogil except +
        bool empty()                      nogil except +







    def __getitem__(self,  in_0 ):
        assert isinstance(in_0, (int, long)), 'arg in_0 wrong type'
    
        cdef long _idx = (<int>in_0)
        if _idx >= self.inst.get().size():
            raise IndexError("invalid index %d" % _idx)

        cdef int _r = deref(self.inst.get())[(<int>in_0)]
        py_result = <int>_r
        return py_result

    def __setitem__(self, key, value):
        assert isinstance(key, (int, long)), 'arg key wrong type'
        assert isinstance(value, (int, long)), 'arg value wrong type'
        cdef long _idx = (<int>key)
        if _idx >= self.inst.get().size():
            raise IndexError("invalid index %d" % _idx)

        cdef int _v = (<int>value)
        deref(self.inst.get())[(<int>key)] = _v

    def get_data(self):
        """
        Gets the raw data for the integer data array

        Example usage: 

          idata = pyopenms.IntegerDataArray()
          data = idata.get_data()

        """

        cdef _IntegerDataArray * ida_ = self.inst.get()
        cdef libcpp_vector[int].iterator it = ida_.begin()
        cdef unsigned int n = ida_.size()
        cdef np.ndarray[int, ndim=1] data
        data = np.zeros( (n,), dtype=np.intc)

        cdef int i = 0
        while it != ida_.end():
            data[i] = deref(it)
            inc(it)
            i += 1

        return data

    def set_data(self, np.ndarray[int, ndim=1, mode="c"] data not None):
        """
        Sets the raw data for the integer data array

        Example usage: 

          idata = pyopenms.IntegerDataArray()
          data = numpy.array( [1, 2, 3, 5 ,6] ).astype(np.intc)
          idata.set_data(data)

        """

        cdef _IntegerDataArray * ida_ = self.inst.get()
        ida_.clear()

        cdef int N
        N = len(data)
        ida_.reserve(N) # allocate space for incoming data
        for i in range(N):
            ida_.push_back(data[i])




from Types cimport *
from Param cimport *
from TransformationModel cimport *
from String cimport *
from StringList cimport *
from TransformationModel cimport TM_DataPoint

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/TransformationDescription.h>" namespace "OpenMS::TransformationDescription":
    
    cdef cppclass TransformationStatistics "OpenMS::TransformationDescription::TransformationStatistics":
        TransformationStatistics() nogil except +
        TransformationStatistics(TransformationStatistics) nogil except +
        # libcpp_vector[ size_t ] percents # const
        double xmin
        double xmax
        double ymin
        double ymax
        libcpp_map[size_t, double ] percentiles_before
        libcpp_map[size_t, double ] percentiles_after

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/TransformationDescription.h>" namespace "OpenMS":

    cdef cppclass TransformationDescription:
        TransformationDescription() nogil except +
        TransformationDescription(TransformationDescription) nogil except + # wrap-ignore
        libcpp_vector[TM_DataPoint] getDataPoints() nogil except +
        void setDataPoints(libcpp_vector[TM_DataPoint]& data) nogil except +
        void setDataPoints(libcpp_vector[libcpp_pair[double,double]]& data) nogil except +
        double apply(double) nogil except +

        void fitModel(String model_type, Param params)  nogil except +
        void fitModel(String model_type)  nogil except +

        String getModelType()  nogil except +
        Param getModelParameters() nogil except +

        void invert() nogil except +

        void getDeviations(libcpp_vector[double]& diffs, bool do_apply, bool do_sort) nogil except +
        TransformationStatistics getStatistics() nogil except +

        # NAMESPACE # void printSummary(std::ostream & os) nogil except +

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/TransformationDescription.h>" namespace "OpenMS::TransformationDescription":

    void getModelTypes(StringList result) nogil except + # wrap-attach:TransformationDescription




from Types cimport *
from Map cimport *
from Types cimport *
from StringList cimport *
from String cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/HiddenMarkovModel.h>" namespace "OpenMS":
    
    cdef cppclass HiddenMarkovModel "OpenMS::HiddenMarkovModel":
        HiddenMarkovModel() nogil except +
        HiddenMarkovModel(HiddenMarkovModel) nogil except +
        void writeGraphMLFile(const String & filename) nogil except +
        # NAMESPACE # void write(std::ostream & out) nogil except +
        double getTransitionProbability(const String & s1, const String & s2) nogil except +
        void setTransitionProbability(const String & s1, const String & s2, double prob) nogil except +
        Size getNumberOfStates() nogil except +
        void addNewState(HMMState * state) nogil except +
        void addNewState(const String & name) nogil except +
        void addSynonymTransition(const String & name1, const String & name2, const String & synonym1, const String & synonym2) nogil except +
        void evaluate() nogil except +
        void train() nogil except +
        void setInitialTransitionProbability(const String & state, double prob) nogil except +
        void clearInitialTransitionProbabilities() nogil except +
        void setTrainingEmissionProbability(const String & state, double prob) nogil except +
        void clearTrainingEmissionProbabilities() nogil except +
        void enableTransition(const String & s1, const String & s2) nogil except +
        void disableTransition(const String & s1, const String & s2) nogil except +
        void disableTransitions() nogil except +
        # POINTER # void calculateEmissionProbabilities(Map[ HMMState *, double ] & emission_probs) nogil except +
        void dump() nogil except +
        void forwardDump() nogil except +
        void estimateUntrainedTransitions() nogil except +
        HMMState * getState(const String & name) nogil except +
        void clear() nogil except +
        void setPseudoCounts(double pseudo_counts) nogil except +
        double getPseudoCounts() nogil except +
        void setVariableModifications(StringList & modifications) nogil except +

cdef extern from "<OpenMS/ANALYSIS/ID/HiddenMarkovModel.h>" namespace "OpenMS":
    
    cdef cppclass HMMState "OpenMS::HMMState":
        HMMState() nogil except +
        HMMState(HMMState) nogil except +
        HMMState(const String & name, bool hidden) nogil except +

        # They dont exist ...
        # bool operator==(HMMState) nogil except +
        # bool operator!=(HMMState) nogil except +

        void setName(const String & name) nogil except +
        String getName() nogil except +
        void setHidden(bool hidden) nogil except +
        bool isHidden() nogil except +
        void addPredecessorState(HMMState * state) nogil except +
        void deletePredecessorState(HMMState * state) nogil except +
        void addSuccessorState(HMMState * state) nogil except +
        void deleteSuccessorState(HMMState * state) nogil except +
        libcpp_set[ HMMState * ]  getPredecessorStates() nogil except +
        libcpp_set[ HMMState * ]  getSuccessorStates() nogil except +







    def getMSLevels(self):

        cdef libcpp_vector[unsigned int] _r = self.inst.get().getMSLevels()
        cdef libcpp_vector[unsigned int].iterator it__r = _r.begin()
        cdef list result = []
        while it__r != _r.end():
            result.append(deref(it__r))
            inc(it__r)
        return result




    def getChromatogram(self,  id_ ):
        """Cython signature: MSChromatogram getChromatogram(size_t id_)"""
        assert isinstance(id_, (int, long)), 'arg id_ wrong type'
        assert id_ < self.getNrChromatograms(), 'Requested chromatogram %s does not exist, there are only %s chromatograms' % (id_, self.getNrChromatograms() )
    
        cdef _MSChromatogram * _r = new _MSChromatogram(self.inst.get().getChromatogram((<size_t>id_)))
        cdef MSChromatogram py_result = MSChromatogram.__new__(MSChromatogram)
        py_result.inst = shared_ptr[_MSChromatogram](_r)
        return py_result

    def getSpectrum(self,  id_ ):
        """Cython signature: MSSpectrum getSpectrum(size_t id_)"""
        assert isinstance(id_, (int, long)), 'arg id_ wrong type'
        assert id_ < self.getNrSpectra(), 'Requested spectrum %s does not exist, there are only %s spectra' % (id_, self.getNrSpectra() )
    
        cdef _MSSpectrum * _r = new _MSSpectrum(self.inst.get().getSpectrum((<size_t>id_)))
        cdef MSSpectrum py_result = MSSpectrum.__new__(MSSpectrum)
        py_result.inst = shared_ptr[_MSSpectrum](_r)
        return py_result



from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from MassDecomposition cimport *
# from RealMassDecomposer cimport *
# from IMSAlphabet cimport *
# from Weights cimport *

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/MassDecompositionAlgorithm.h>" namespace "OpenMS":
    
    cdef cppclass MassDecompositionAlgorithm(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        MassDecompositionAlgorithm() nogil except +
        MassDecompositionAlgorithm(MassDecompositionAlgorithm) nogil except + #wrap-ignore
        void getDecompositions(libcpp_vector[ MassDecomposition ] & decomps, double weight) nogil except +








    def encode64(self, list in_ , int to_byte_order ,  out ,  zlib_compression ):
        """Cython signature: void encode64(libcpp_vector[double] & in_, ByteOrder to_byte_order, String & out, bool zlib_compression)"""
        assert isinstance(in_, list) and all(isinstance(elemt_rec, float) for elemt_rec in in_), 'arg in_ wrong type'
        assert to_byte_order in [0, 1], 'arg to_byte_order wrong type'
        assert isinstance(out, String), 'arg out wrong type'
        assert isinstance(zlib_compression, (int, long)), 'arg zlib_compression wrong type'
        cdef libcpp_vector[double] v0 = in_
    
    
    
        self.inst.get().encode(v0, (<_ByteOrder>to_byte_order), deref((<String>out).inst.get()), (<bool>zlib_compression))
    
    def decode64(self,  in_ , int from_byte_order , list out ,  zlib_compression ):
        """Cython signature: void decode64(const String & in_, ByteOrder from_byte_order, libcpp_vector[double] & out, bool zlib_compression)"""
        assert (isinstance(in_, str) or isinstance(in_, unicode) or isinstance(in_, bytes) or isinstance(in_, String)), 'arg in_ wrong type'
        assert from_byte_order in [0, 1], 'arg from_byte_order wrong type'
        assert isinstance(out, list) and all(isinstance(elemt_rec, float) for elemt_rec in out), 'arg out wrong type'
        assert isinstance(zlib_compression, (int, long)), 'arg zlib_compression wrong type'
    
    
        cdef libcpp_vector[double] v2 = out
    
        self.inst.get().decode(deref((convString(in_)).get()), (<_ByteOrder>from_byte_order), v2, (<bool>zlib_compression))
        out[:] = v2

    def encode32(self, list in_ , int to_byte_order ,  out ,  zlib_compression ):
        """Cython signature: void encode32(libcpp_vector[float] & in_, ByteOrder to_byte_order, String & out, bool zlib_compression)"""
        assert isinstance(in_, list) and all(isinstance(elemt_rec, float) for elemt_rec in in_), 'arg in_ wrong type'
        assert to_byte_order in [0, 1], 'arg to_byte_order wrong type'
        assert isinstance(out, String), 'arg out wrong type'
        assert isinstance(zlib_compression, (int, long)), 'arg zlib_compression wrong type'
        cdef libcpp_vector[float] v0 = in_
    
    
    
        self.inst.get().encode(v0, (<_ByteOrder>to_byte_order), deref((<String>out).inst.get()), (<bool>zlib_compression))
    
    def decode32(self,  in_ , int from_byte_order , list out ,  zlib_compression ):
        """Cython signature: void decode32(const String & in_, ByteOrder from_byte_order, libcpp_vector[float] & out, bool zlib_compression)"""
        assert (isinstance(in_, str) or isinstance(in_, unicode) or isinstance(in_, bytes) or isinstance(in_, String)), 'arg in_ wrong type'
        assert from_byte_order in [0, 1], 'arg from_byte_order wrong type'
        assert isinstance(out, list) and all(isinstance(elemt_rec, float) for elemt_rec in out), 'arg out wrong type'
        assert isinstance(zlib_compression, (int, long)), 'arg zlib_compression wrong type'
    
    
        cdef libcpp_vector[float] v2 = out
    
        self.inst.get().decode(deref((convString(in_)).get()), (<_ByteOrder>from_byte_order), v2, (<bool>zlib_compression))
        out[:] = v2



from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool

from Types cimport *
from MetaInfoInterface cimport *

cdef extern from "<OpenMS/METADATA/IdentificationHit.h>" namespace "OpenMS":

    cdef cppclass IdentificationHit(MetaInfoInterface):
        # wrap-inherits:
        #   MetaInfoInterface

        IdentificationHit()   nogil except +
        IdentificationHit(IdentificationHit) nogil except + # wrap-ignore

        # /// sets the identifier
        void setId(String id) nogil except +

        # /// returns the id
        String getId() nogil except +

        # /// sets the charge state of the peptide
        void setCharge(Int charge) nogil except +

        # /// returns the charge state
        Int getCharge() nogil except +

        # /// sets the calculated mass to charge ratio
        void setCalculatedMassToCharge(double mz) nogil except +

        # /// returns the calculated mass to charge ratio
        double getCalculatedMassToCharge() nogil except +

        # /// sets the experimental mass to charge ratio
        void setExperimentalMassToCharge(double mz) nogil except +

        # /// returns the experimental mass to charge
        double getExperimentalMassToCharge() nogil except +

        # /// sets the name
        void setName(String name) nogil except +

        # /// returns the name
        String getName() nogil except +

        # /// sets whether the peptide passed the threshold
        void setPassThreshold(bool) nogil except +

        # /// returns whether the peptide passed the threshold
        bool getPassThreshold() nogil except +

        # /// set the rank of the peptide
        void setRank(Int rank) nogil except +

        # /// returns the rank of the peptide
        Int getRank() nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
# from libcpp.map cimport map as libcpp_map
from libcpp cimport bool
from MSSpectrum cimport *
from MSExperiment cimport *
from Precursor cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/PrecursorPurity.h>" namespace "OpenMS":

    cdef cppclass PrecursorPurity "OpenMS::PrecursorPurity":

        PrecursorPurity() nogil except +
        PrecursorPurity(PrecursorPurity) nogil except + # wrap-ignore

        # libcpp_map[String, PurityScores] computePrecursorPurities(MSExperiment spectra,
        #                                                     double precursor_mass_tolerance,
        #                                                     bool precursor_mass_tolerance_unit_ppm) nogil except +

        PurityScores computePrecursorPurity(MSSpectrum ms1,
                                            Precursor pre,
                                            double precursor_mass_tolerance,
                                            bool precursor_mass_tolerance_unit_ppm) nogil except +

    cdef cppclass PurityScores "OpenMS::PrecursorPurity::PurityScores":

        PurityScores(PurityScores) nogil except +
        PurityScores() nogil except +

        double total_intensity
        double target_intensity
        double signal_proportion
        Size target_peak_count
        Size residual_peak_count



from ConvexHull2D cimport ConvexHull2D as _ConvexHull2D
from DPosition cimport DPosition2 as _DPosition2
from DBoundingBox cimport DBoundingBox2 as _DBoundingBox2


    def enclosesXY(self, float x, float y):
        """
        Parameters:
        x (float)
        y (float)
        
        Returns:
        int
        """
        cdef _DPosition2 pos
        pos[0] = x
        pos[1] = y
        return self.inst.get().encloses(pos)


    def getHullPointsNPY(self):
        """
        Returns:
        result (np.ndarray[np.float32_t, ndim=2])
        """
        cdef libcpp_vector[_DPosition2] points = self.inst.get().getHullPoints()
        cdef np.ndarray[np.float32_t, ndim=2] result
        cdef n = points.size()
        result = np.zeros( [n,2], dtype=np.float32)
        cdef libcpp_vector[_DPosition2].iterator it = points.begin()
        cdef int i = 0
        while it != points.end():
            result[i,0] = deref(it)[0]
            result[i,1] = deref(it)[1]
            inc(it)
            i += 1
        return result

    def setHullPointsNPY(self, np.ndarray[np.float32_t, ndim=2] points):
        """
        Parameters:
        points (np.ndarray[np.float32_t, ndim=2])
        """
        cdef _ConvexHull2D * hull = self.inst.get()
        cdef int N = points.shape[0]
        cdef int i
        cdef libcpp_vector[_DPosition2] vec
        cdef _DPosition2 p
        for i in range(N):
            p[0] = points[i,0]
            p[1] = points[i,1]
            vec.push_back(p)
        self.inst.get().setHullPoints(vec)

    def getBoundingBox2D(self):
        """
        Returns:
        ((double,double),(double,double))
        """
        cdef _DBoundingBox2 box = self.inst.get().getBoundingBox()
        cdef _DPosition2 minp = box.minPosition()
        cdef _DPosition2 maxp = box.maxPosition()
        return (minp[0], minp[1]), (maxp[0], maxp[1])

    def addPointXY(self, x, y):
        """
        Parameters:
        x (double)
        y (double)
        """
        cdef _DPosition2 p
        p[0] = x
        p[1] = y
        self.inst.get().addPoint(p)

    def addPointsNPY(self, np.ndarray[np.float32_t, ndim=2] points):
        """
        Parameters:
        points (np.ndarray[np.float32_t, ndim=2])
        """
        cdef _ConvexHull2D * hull = self.inst.get()
        cdef int N = points.shape[0]
        cdef int i
        cdef libcpp_vector[_DPosition2] vec
        cdef _DPosition2 p
        for i in range(N):
            p[0] = points[i,0]
            p[1] = points[i,1]
            vec.push_back(p)
        self.inst.get().addPoints(vec)




from Types cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/ANALYSIS/XLMS/OPXLDataStructs.h>" namespace "OpenMS::OPXLDataStructs":

    cdef cppclass OPXL_PreprocessedPairSpectra "OpenMS::OPXLDataStructs::PreprocessedPairSpectra":

        OPXL_PreprocessedPairSpectra(OPXL_PreprocessedPairSpectra) nogil except +
        OPXL_PreprocessedPairSpectra() nogil except + # wrap-ignore
        OPXL_PreprocessedPairSpectra(Size size) nogil except +

        MSExperiment spectra_linear_peaks
        MSExperiment spectra_xlink_peaks
        MSExperiment spectra_all_peaks




import pygraphviz as pgv
import sys
import pylab as P
#P.ion()

def main(argv):
    if len(argv) != 2:
        print 'Usage: draw_dot.py <graph .dot> <graph .png output>'
    else:
        G = pgv.AGraph(argv[0])

        G.layout(prog='circo')
        G.layout(prog='neato', args='-Goverlap=scale -Gsplines=true')
        G.draw(argv[1])

if __name__ == "__main__":
    main(sys.argv[1:])



from MSSpectrum cimport *
from IsotopeDistribution cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/IsotopeWavelet.h>" namespace "OpenMS":
    
    cdef cppclass IsotopeWavelet "OpenMS::IsotopeWavelet":
        IsotopeWavelet(IsotopeWavelet) nogil except + #wrap-ignore

        # IsotopeWavelet * init(double max_m, UInt max_charge) nogil except +
        # IsotopeWavelet * getInstance() nogil except +
        void destroy() nogil except +
        double getValueByMass(double t, double m, UInt z, Int mode) nogil except +
        double getValueByLambda(double lambda_, double tz1) nogil except +
        double getValueByLambdaExtrapol(double lambda_, double tz1) nogil except +
        double getValueByLambdaExact(double lambda_, double tz1) nogil except +
        UInt getMaxCharge() nogil except +
        void setMaxCharge(UInt max_charge) nogil except +
        double getTableSteps() nogil except +
        double getInvTableSteps() nogil except +
        void setTableSteps(double table_steps) nogil except +
        double getLambdaL(double m) nogil except +
        # IsotopeDistribution::ContainerType  getAveragine(double m, UInt *size) nogil except +
        Size getGammaTableMaxIndex() nogil except +
        Size getExpTableMaxIndex() nogil except +
        float myPow(float a, float b) nogil except +
        UInt getMzPeakCutOffAtMonoPos(double mass, UInt z) nogil except +
        UInt getNumPeakCutOff(double mass, UInt z) nogil except +
        UInt getNumPeakCutOff(double mz) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from ProgressLogger cimport *
# from DistanceMatrix cimport *
# from ClusterFunctor cimport *

cdef extern from "<OpenMS/COMPARISON/CLUSTERING/AverageLinkage.h>" namespace "OpenMS":
    
    cdef cppclass AverageLinkage "OpenMS::AverageLinkage":
        AverageLinkage() nogil except +
        AverageLinkage(AverageLinkage) nogil except +
        # void operator()(DistanceMatrix[ float ] &original_distance, libcpp_vector[ BinaryTreeNode ] &cluster_tree, float threshold) nogil except +
        # ClusterFunctor * create() nogil except +
        String getProductName() nogil except +




from Types cimport *
from String cimport *
from InstrumentSettings cimport *
from SourceFile cimport *
from PeptideIdentification cimport *
from DataProcessing cimport *
from Product cimport *
from AcquisitionInfo cimport *
from Precursor cimport *
from MetaInfoInterface cimport *

cdef extern from "<OpenMS/METADATA/ChromatogramSettings.h>" namespace "OpenMS":

    cdef cppclass ChromatogramSettings(MetaInfoInterface):
        # wrap-inherits:
        #    MetaInfoInterface
        #
        # wrap-doc:
        #   Description of the chromatogram settings, provides meta-information
        #   about a single chromatogram.

        ChromatogramSettings() nogil except +
        ChromatogramSettings(ChromatogramSettings) nogil except +

        Product getProduct() nogil except + # wrap-doc:Access to the product ion
        void setProduct(Product p) nogil except + # wrap-doc:Set the product ion

        String getNativeID() nogil except + # wrap-doc:returns the native identifier for the spectrum, used by the acquisition software.
        void setNativeID(String native_id) nogil except + # wrap-doc:sets the native identifier for the spectrum, used by the acquisition software.

        # returns the free-text comment
        String getComment() nogil except +
        # sets the free-text comment
        void setComment(String comment) nogil except +

        # returns a mutable reference to the instrument settings of the current spectrum
        InstrumentSettings getInstrumentSettings() nogil except +
        # sets the instrument settings of the current spectrum
        void setInstrumentSettings(InstrumentSettings instrument_settings) nogil except +

        # returns a mutable reference to the acquisition info
        AcquisitionInfo getAcquisitionInfo() nogil except +
        # sets the acquisition info
        void setAcquisitionInfo(AcquisitionInfo acquisition_info) nogil except +

        # returns a mutable reference to the source file
        SourceFile getSourceFile() nogil except +
        # sets the source file
        void setSourceFile(SourceFile source_file) nogil except +

        # returns a mutable reference to the precursors
        Precursor getPrecursor() nogil except +
        # sets the precursors
        void setPrecursor(Precursor precursor) nogil except +

        # returns a mutable reference to the description of the applied processing
        libcpp_vector[ shared_ptr[DataProcessing] ] getDataProcessing() nogil except +
        # sets the description of the applied processing
        void setDataProcessing(libcpp_vector[ shared_ptr[DataProcessing] ])   nogil except +

        # sets the chromatogram type
        void setChromatogramType(ChromatogramType type) nogil except +
        ChromatogramType getChromatogramType() nogil except +


cdef extern from "<OpenMS/METADATA/ChromatogramSettings.h>" namespace "OpenMS::ChromatogramSettings":

    cdef enum ChromatogramType:
        # wrap-attach:
        #     ChromatogramSettings

        MASS_CHROMATOGRAM,
        TOTAL_ION_CURRENT_CHROMATOGRAM,
        SELECTED_ION_CURRENT_CHROMATOGRAM,
        BASEPEAK_CHROMATOGRAM,
        SELECTED_ION_MONITORING_CHROMATOGRAM,
        SELECTED_REACTION_MONITORING_CHROMATOGRAM,
        ELECTROMAGNETIC_RADIATION_CHROMATOGRAM,
        ABSORPTION_CHROMATOGRAM,
        EMISSION_CHROMATOGRAM,
        SIZE_OF_CHROMATOGRAM_TYPE








    # NOTE: using shared_ptr for a singleton will lead to segfaults, use raw ptr instead
    # cdef AutowrapPtrHolder[_RibonucleotideDB] inst

    def __init__(self):
      self.inst = AutowrapPtrHolder[_RibonucleotideDB](_getInstance_RibonucleotideDB())

    def __dealloc__(self):
      # Careful here, the wrapped ptr is a single instance and we should not
      # reset it, therefore use 'wrap-manual-dealloc'
      pass


    def getRibonucleotideAlternatives(self, bytes code ):
        """Cython signature: libcpp_pair[const Ribonucleotide *,const Ribonucleotide *] getRibonucleotideAlternatives(const libcpp_string & code)"""
        assert isinstance(code, bytes), 'arg code wrong type'
    
        _r = self.inst.get().getRibonucleotideAlternatives((<libcpp_string>code))
        cdef const _Ribonucleotide * out_ptr1 = _r.first
        cdef const _Ribonucleotide * out_ptr2 = _r.second

        cdef Ribonucleotide out1 = Ribonucleotide.__new__(Ribonucleotide)
        cdef Ribonucleotide out2 = Ribonucleotide.__new__(Ribonucleotide)

        out1.inst = shared_ptr[_Ribonucleotide](new _Ribonucleotide(deref(_r.first)))
        out2.inst = shared_ptr[_Ribonucleotide](new _Ribonucleotide(deref(_r.second)))

        cdef list py_result = [out1, out2]
        return py_result 



from libcpp cimport bool

from CVMappings cimport *
from String cimport *

cdef extern from "<OpenMS/FORMAT/CVMappingFile.h>" namespace "OpenMS":

    cdef cppclass CVMappingFile:

        CVMappingFile() nogil except +
        # CVMappingFile(CVMappingFile) nogil except + # private
        void load(const String & filename, CVMappings & cv_mappings, bool strip_namespaces) nogil except +




from Types cimport *
from String cimport *
from EmpiricalFormula cimport *


cdef extern from "<OpenMS/CHEMISTRY/Ribonucleotide.h>" namespace "OpenMS::Ribonucleotide":

    cdef enum TermSpecificityNuc:
      # wrap-attach:
      # Ribonucleotide
      ANYWHERE = 0,
      FIVE_PRIME,
      THREE_PRIME,
      NUMBER_OF_TERM_SPECIFICITY


cdef extern from "<OpenMS/CHEMISTRY/Ribonucleotide.h>" namespace "OpenMS":

    cdef cppclass Ribonucleotide:
        # wrap-hash:
        #   getName().c_str()

        Ribonucleotide() nogil except +
        Ribonucleotide(Ribonucleotide) nogil except + # wrap-ignore

        # detailed constructor
        Ribonucleotide(String name,
                String code,
                String new_code,
                String html_code,
                EmpiricalFormula formula,
                char origin,
                double mono_mass,
                double avg_mass,
                TermSpecificityNuc term_spec,
                EmpiricalFormula baseloss_formula) nogil except +

 
        # return the short name
        String getCode() nogil except +

        # set the short name
        void setCode(String code) nogil except +

        # sets the name of the ribonucleotide
        void setName(String name) nogil except +

        # returns the name of the ribonucleotide
        String getName() nogil except +

        # set empirical formula of the ribonucleotide (must be full, with N and C-terminus)
        void setFormula(EmpiricalFormula formula) nogil except +

        # returns the empirical formula of the residue
        EmpiricalFormula getFormula() nogil except +

        # sets average mass of the ribonucleotide
        void setAvgMass(double avg_mass) nogil except +

        # returns average mass of the ribonucleotide
        double getAvgMass() nogil except +

        # sets monoisotopic mass of the ribonucleotide
        void setMonoMass(double mono_mass) nogil except +

        # returns monoisotopic mass of the ribonucleotide
        double getMonoMass() nogil except +

        # return the new code
        String getNewCode() nogil except +

        # set the new code
        void setNewCode(String code) nogil except +

        # Get the code of the unmodified base (e.g., "A", "C", ...)
        char getOrigin() nogil except +

        # Set the code of the unmodified base (e.g., "A", "C", ...)
        void setOrigin(char origin) nogil except +

        # Set the HTML (RNAMods) code
        void setHTMLCode(String html_code) nogil except +

        # Get the HTML (RNAMods) code
        String getHTMLCode() nogil except +

        # Set the terminal specificity
        void setTermSpecificity(TermSpecificityNuc term_spec) nogil except +

        # Get the terminal specificity
        TermSpecificityNuc getTermSpecificity() nogil except +

        # Get sum formula after loss of the nucleobase
        EmpiricalFormula getBaselossFormula() nogil except +

        # Set sum formula after loss of the nucleobase
        void setBaselossFormula(EmpiricalFormula formula) nogil except +

        # equality operator
        bool operator==(Ribonucleotide & ribonucleotide) nogil except +

        # true if the ribonucleotide is a modified one
        bool isModified() nogil except +



from Types cimport *
from String cimport *
from LightTargetedExperiment cimport *
from FeatureMap cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/OpenSwathOSWWriter.h>" namespace "OpenMS":
    
    cdef cppclass OpenSwathOSWWriter "OpenMS::OpenSwathOSWWriter":

        OpenSwathOSWWriter(OpenSwathOSWWriter) nogil except + #wrap-ignore
        OpenSwathOSWWriter(String output_filename, String input_filename, bool ms1_scores, bool sonar, bool uis_scores) nogil except +

        bool isActive() nogil except +
        void writeHeader() nogil except +
        String prepareLine(LightCompound & compound, LightTransition * tr, FeatureMap & output, String id_) nogil except +
        void writeLines(libcpp_vector[ String ] to_osw_output) nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from String cimport *

cdef extern from "<OpenMS/FORMAT/Base64.h>" namespace "OpenMS":
    
    cdef cppclass Base64 "OpenMS::Base64":
        Base64() nogil except +
        Base64(Base64) nogil except + #wrap-ignore

        void encode(libcpp_vector[ double ] & in_, ByteOrder to_byte_order, String &out, bool zlib_compression) nogil except +#wrap-ignore
        void decode(const String & in_, ByteOrder from_byte_order, libcpp_vector[ double ] &out, bool zlib_compression) nogil except +#wrap-ignore
        void encode(libcpp_vector[ float ] & in_, ByteOrder to_byte_order, String &out, bool zlib_compression) nogil except +#wrap-ignore
        void decode(const String & in_, ByteOrder from_byte_order, libcpp_vector[ float ] &out, bool zlib_compression) nogil except +#wrap-ignore

        void encode64(libcpp_vector[ double ] & in_, ByteOrder to_byte_order, String &out, bool zlib_compression) nogil except + #wrap-ignore
        void decode64(const String & in_, ByteOrder from_byte_order, libcpp_vector[ double ] &out, bool zlib_compression) nogil except + #wrap-ignore
        void encode32(libcpp_vector[ float ] & in_, ByteOrder to_byte_order, String &out, bool zlib_compression) nogil except + #wrap-ignore
        void decode32(const String & in_, ByteOrder from_byte_order, libcpp_vector[ float ] &out, bool zlib_compression) nogil except + #wrap-ignore

        void encodeIntegers(libcpp_vector[ int ] & in_, ByteOrder to_byte_order, String &out, bool zlib_compression) nogil except +
        void decodeIntegers(const String & in_, ByteOrder from_byte_order, libcpp_vector[ int ] &out, bool zlib_compression) nogil except +

        void encodeStrings(libcpp_vector[ String ] & in_, String &out, bool zlib_compression) nogil except +
        void decodeStrings(const String & in_, libcpp_vector[ String ] &out, bool zlib_compression) nogil except +

        # void decodeSingleString(const String & in, QByteArray & base64_uncompressed, bool zlib_compression);

cdef extern from "<OpenMS/FORMAT/Base64.h>" namespace "OpenMS::Base64":
    cdef enum ByteOrder "OpenMS::Base64::ByteOrder":
        #wrap-attach:
        #    Base64
        BYTEORDER_BIGENDIAN
        BYTEORDER_LITTLEENDIAN



cimport numpy as np
import numpy as np




    def get_peaks(self):

        cdef _MSChromatogram * chrom_ = self.inst.get()

        cdef unsigned int n = chrom_.size()
        cdef np.ndarray[np.float64_t, ndim=1] rts
        rts = np.zeros( (n,), dtype=np.float64)
        cdef np.ndarray[np.float32_t, ndim=1] intensities
        intensities = np.zeros( (n,), dtype=np.float32)
        cdef _ChromatogramPeak p

        cdef libcpp_vector[_ChromatogramPeak].iterator it = chrom_.begin()
        cdef int i = 0
        while it != chrom_.end():
            rts[i] = deref(it).getRT()
            intensities[i] = deref(it).getIntensity()
            inc(it)
            i += 1

        return rts, intensities

    def set_peaks(self, peaks):

        assert isinstance(peaks, (tuple, list)), "Input for set_peaks needs to be a tuple or a list of size 2 (rt and intensity vector)"
        assert len(peaks) == 2, "Input for set_peaks needs to be a tuple or a list of size 2 (rt and intensity vector)"

        rts, intensities = peaks
        assert len(rts) == len(intensities), "Input vectors for set_peaks need to have the same length (rt and intensity vector)"

        # Select which function to use for set_peaks:
        # If we have numpy arrays, it helps to use optimized functions
        if isinstance(rts, np.ndarray) and isinstance(intensities, np.ndarray) and \
          rts.dtype == np.float64 and intensities.dtype == np.float32 and \
          rts.flags["C_CONTIGUOUS"] and intensities.flags["C_CONTIGUOUS"]  :
            self._set_peaks_fast_df(rts, intensities)
        elif isinstance(rts, np.ndarray) and isinstance(intensities, np.ndarray) and \
          rts.dtype == np.float64 and intensities.dtype == np.float64 and \
          rts.flags["C_CONTIGUOUS"] and intensities.flags["C_CONTIGUOUS"]  :
            self._set_peaks_fast_dd(rts, intensities)
        else:
            self._set_peaks_orig(rts, intensities)



    def _set_peaks_fast_dd(self, np.ndarray[double, ndim=1, mode="c"] data_rt not None, np.ndarray[double, ndim=1, mode="c"] data_i not None):

        cdef _MSChromatogram * chrom_ = self.inst.get()

        chrom_.clear(0) # empty vector , keep meta data
        chrom_.reserve(<int>len(data_rt)) # allocate space for incoming data
        cdef _ChromatogramPeak p = _ChromatogramPeak()
        cdef double rt
        cdef double intensity
        cdef int N
        N = len(data_rt)

        for i in range(N):
            rt = data_rt[i]
            intensity = data_i[i]
            p.setRT(<double>rt)
            p.setIntensity(<float>intensity)
            chrom_.push_back(p)

        chrom_.updateRanges()


    def _set_peaks_fast_df(self, np.ndarray[double, ndim=1, mode="c"] data_rt not None, np.ndarray[float, ndim=1, mode="c"] data_i not None):

        cdef _MSChromatogram * chrom_ = self.inst.get()

        chrom_.clear(0) # empty vector , keep meta data
        chrom_.reserve(<int>len(data_rt)) # allocate space for incoming data
        cdef _ChromatogramPeak p = _ChromatogramPeak()
        cdef double rt
        cdef float intensity
        cdef int N
        N = len(data_rt)

        for i in range(N):
            rt = data_rt[i]
            intensity = data_i[i]
            p.setRT(<double>rt)
            p.setIntensity(<float>intensity)
            chrom_.push_back(p)

        chrom_.updateRanges()


    def _set_peaks_orig(self, rts, intensities):


        cdef _MSChromatogram * chrom_ = self.inst.get()

        chrom_.clear(0) # empty vector , keep meta data
        chrom_.reserve(<int>len(rts)) # allocate space for incoming data
        cdef _ChromatogramPeak p = _ChromatogramPeak()
        cdef double rt
        cdef float intensity
        cdef int N
        N = len(rts)

        for i in range(N):
            rt = rts[i]
            intensity = intensities[i]
            p.setRT(<double>rt)
            p.setIntensity(<float>intensity)
            chrom_.push_back(p)

        chrom_.updateRanges()




from Types cimport *
from libcpp cimport bool
from String cimport *
from Map cimport *
from Element cimport *
from IsotopeDistribution cimport *

cdef extern from "<OpenMS/CHEMISTRY/ElementDB.h>" namespace "OpenMS":
    
    cdef cppclass ElementDB "OpenMS::ElementDB":
        # wrap-manual-memory:
        #    cdef AutowrapConstPtrHolder[_ElementDB] inst

        ElementDB(ElementDB) nogil except + #wrap-ignore

        # No wrapping of const ref
        # const Map[ String, Element * ]  getNames() nogil except +
        # const Map[ String, Element * ] getSymbols() nogil except +
        # const Map[unsigned int, Element * ] getAtomicNumbers() nogil except +
        const Element * getElement(const String & name) nogil except +
        const Element * getElement(UInt atomic_number) nogil except +
        bool hasElement(const String & name) nogil except +
        bool hasElement(UInt atomic_number) nogil except +

## wrap static methods
cdef extern from "<OpenMS/CHEMISTRY/ElementDB.h>" namespace "OpenMS::ElementDB":
    
    const ElementDB* getInstance() nogil except + # wrap-ignore




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool
from EmgFitter1D cimport *
from EmgModel cimport *
from GaussFilter cimport *
from MRMFeature cimport *
from MRMTransitionGroup cimport *
from DPosition cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/EmgScoring.h>" namespace "OpenMS":
    
    cdef cppclass EmgScoring "OpenMS::EmgScoring":
        EmgScoring() nogil except +
        EmgScoring(EmgScoring) nogil except + #wrap-ignore

        void setFitterParam(Param param) nogil except +
        Param getDefaults() nogil except +
        # TEMPLATE # double calcElutionFitScore(MRMFeature &mrmfeature, MRMTransitionGroup[ SpectrumType, TransitionT ] &transition_group) nogil except +
        double elutionModelFit(libcpp_vector[DPosition2] current_section, bool smooth_data) nogil except +




from Types cimport *
from IsobaricQuantitationMethod cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/TMTSixPlexQuantitationMethod.h>" namespace "OpenMS":
    
    cdef cppclass TMTSixPlexQuantitationMethod(IsobaricQuantitationMethod) :
        # wrap-inherits:
        #  IsobaricQuantitationMethod
        TMTSixPlexQuantitationMethod() nogil except +
        TMTSixPlexQuantitationMethod(TMTSixPlexQuantitationMethod) nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from DPosition cimport *
from DBoundingBox cimport *

# this class has addons, see the ./addons folder

cdef extern from "<OpenMS/DATASTRUCTURES/ConvexHull2D.h>" namespace "OpenMS":

    cdef cppclass ConvexHull2D:

        ConvexHull2D() nogil except +
        ConvexHull2D(ConvexHull2D) nogil except + # wrap-ignore

        bool operator==(ConvexHull2D) nogil except +
        void clear()       nogil except +
        Size compress()    nogil except +
        void expandToBoundingBox() nogil except +

        # as cython/autowrap have problems handling integer template args
        # we wrap following methods manually in ../addons/ConvexHull2D.pyx:
        # better/todo: implement converter in ../converters !

        bool addPoint(DPosition2 point) nogil except +
        void addPoints(libcpp_vector[DPosition2] points) nogil except +

        bool encloses(DPosition2) nogil except +
        libcpp_vector[DPosition2] getHullPoints() nogil except + 
        void setHullPoints(libcpp_vector[DPosition2] ) nogil except + 
        DBoundingBox2 getBoundingBox() nogil except +




from MetaInfoInterface cimport *
from IonSource cimport *
from ScanWindow cimport *

cdef extern from "<OpenMS/METADATA/InstrumentSettings.h>" namespace "OpenMS":

    cdef cppclass InstrumentSettings(MetaInfoInterface):
        # wrap-inherits:
        #    MetaInfoInterface

        InstrumentSettings()     nogil except +
        InstrumentSettings(InstrumentSettings)     nogil except +

        Polarity getPolarity()     nogil except +
        void setPolarity(Polarity)  nogil except +

        ScanMode getScanMode() nogil except +
        void setScanMode(ScanMode scan_mode) nogil except +
        bool getZoomScan() nogil except +
        void setZoomScan(bool zoom_scan) nogil except +
        libcpp_vector[ ScanWindow ]  getScanWindows() nogil except +
        void setScanWindows(libcpp_vector[ ScanWindow ] scan_windows) nogil except +

cdef extern from "<OpenMS/METADATA/InstrumentSettings.h>" namespace "OpenMS::InstrumentSettings":

    # scan mode
    cdef enum ScanMode:
      UNKNOWN,                #< Unknown scan method
      MASSSPECTRUM,           #< general spectrum type
      MS1SPECTRUM,            #< full scan mass spectrum, is a "mass spectrum" @n Synonyms: 'full spectrum', 'Q1 spectrum', 'Q3 spectrum', 'Single-Stage Mass Spectrometry'
      MSNSPECTRUM,            #< MS2+ mass spectrum, is a "mass spectrum"
      SIM,                    #< Selected ion monitoring scan @n Synonyms: 'Multiple ion monitoring scan', 'SIM scan', 'MIM scan'
      SRM,                    #< Selected reaction monitoring scan @n Synonyms: 'Multiple reaction monitoring scan', 'SRM scan', 'MRM scan'
      CRM,                    #< Consecutive reaction monitoring scan @n Synonyms: 'CRM scan'
      CNG,                    #< Constant neutral gain scan @n Synonyms: 'CNG scan'
      CNL,                    #< Constant neutral loss scan @n Synonyms: 'CNG scan'
      PRECURSOR,              #< Precursor ion scan
      EMC,                    #< Enhanced multiply charged scan
      TDF,                    #< Time-delayed fragmentation scan
      EMR,                    #< Electromagnetic radiation scan @n Synonyms: 'EMR spectrum'
      EMISSION,               #< Emission scan
      ABSORPTION,             #< Absorption scan
      SIZE_OF_SCANMODE





from MSExperiment  cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from String cimport *
from StringList cimport *
from ProgressLogger cimport *
from PeakFileOptions cimport *

cdef extern from "<OpenMS/FORMAT/MzDataFile.h>" namespace "OpenMS":

    cdef cppclass MzDataFile(ProgressLogger):
        # wrap-inherits:
        #   ProgressLogger

        MzDataFile() nogil except +

        void load(const String&, MSExperiment &) nogil except+
        void store(const String&, MSExperiment &) nogil except+

        PeakFileOptions getOptions() nogil except +
        void setOptions(PeakFileOptions) nogil except +

        bool isSemanticallyValid(const String& filename, StringList & errors, StringList & warnings) nogil except +





import pyopenms

def convert_to_row(first, targ, run_id, keys, filename):
    peptide_ref = first.getMetaValue("PeptideRef")
    pep = targ.getPeptideByRef(peptide_ref)
    full_peptide_name = "NA"
    if (pep.metaValueExists("full_peptide_name")):
      full_peptide_name = pep.getMetaValue("full_peptide_name")

    decoy = "0"
    peptidetransitions = [t for t in targ.getTransitions() if t.getPeptideRef() == peptide_ref]
    if len(peptidetransitions) > 0:
        if peptidetransitions[0].getDecoyTransitionType() == pyopenms.DecoyTransitionType().DECOY:
            decoy = "1"
        elif peptidetransitions[0].getDecoyTransitionType() == pyopenms.DecoyTransitionType().TARGET:
            decoy = "0"

    protein_name = "NA"
    if len(pep.protein_refs) > 0:
      protein_name = pep.protein_refs[0]

    row = [
        first.getMetaValue("PeptideRef"),
        run_id,
        filename,
        first.getRT(),
        first.getMetaValue("PrecursorMZ"),
        first.getUniqueId(),
        pep.sequence,
        full_peptide_name,
        pep.getChargeState(),
        first.getMetaValue("PrecursorMZ"),
        first.getIntensity(),
        protein_name,
        decoy
    ]

    for k in keys:
        row.append(first.getMetaValue(k))

    return row

def get_header(features):
    keys = []
    features[0].getKeys(keys)
    header = [
        "transition_group_id",
        "run_id",
        "filename",
        "RT",
        "id",
        "Sequence" ,
        "FullPeptideName",
        "Charge",
        "m/z",
        "Intensity",
        "ProteinName",
        "decoy"]
    header.extend(keys)
    return header



from Types cimport *
from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from libcpp.pair cimport pair as libcpp_pair
from libcpp.set cimport set as libcpp_set
from libcpp.vector cimport vector as libcpp_vector
from FeatureMap cimport *
from MSExperiment cimport *
from LPWrapper cimport *
from String cimport *
from PrecursorIonSelectionPreprocessing cimport *
from PSProteinInference cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/ANALYSIS/TARGETED/PSLPFormulation.h>" namespace "OpenMS":
    
    cdef cppclass PSLPFormulation(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler

        PSLPFormulation() nogil except +
        PSLPFormulation(PSLPFormulation) nogil except + #wrap-ignore

        void createAndSolveILPForKnownLCMSMapFeatureBased(
            FeatureMap & features,
            MSExperiment & experiment,
            libcpp_vector[ IndexTriple ] & variable_indices,
            libcpp_vector[ libcpp_vector[ libcpp_pair[ size_t, size_t ] ] ] & mass_ranges, 
            libcpp_set[ int ] & charges_set,
            UInt ms2_spectra_per_rt_bin,
            libcpp_vector[ int ] & solution_indices) nogil except + 

        void createAndSolveILPForInclusionListCreation(
            PrecursorIonSelectionPreprocessing & preprocessing,
            UInt ms2_spectra_per_rt_bin, UInt max_list_size, FeatureMap & precursors, bool solve_ILP) nogil except +

        void createAndSolveCombinedLPForKnownLCMSMapFeatureBased(FeatureMap & features, MSExperiment & experiment, 
            libcpp_vector[ IndexTriple ] & variable_indices, libcpp_vector[ int ] & solution_indices, 
            libcpp_vector[ libcpp_vector[ libcpp_pair[ size_t, size_t ] ] ] & mass_ranges, 
            libcpp_set[ Int ] & charges_set, UInt ms2_spectra_per_rt_bin, Size step_size, bool sequential_order) nogil except + # wrap-ignore

        void updateStepSizeConstraint(Size iteration, UInt step_size) nogil except +

        void updateFeatureILPVariables(
          FeatureMap & new_features,
          libcpp_vector[ IndexTriple ] & variable_indices,
          libcpp_map[ Size, libcpp_vector[ String ] ] & feature_constraints_map) nogil except + # wrap-ignore

        void updateRTConstraintsForSequentialILP(Size & rt_index, UInt ms2_spectra_per_rt_bin, Size max_rt_index) nogil except +

        void updateCombinedILP(FeatureMap & features,
                               PrecursorIonSelectionPreprocessing & preprocessed_db,
                               libcpp_vector[ IndexTriple ] & variable_indices,
                               libcpp_vector[ String ] & new_protein_accs,
                               libcpp_vector[ String ] & protein_accs,
                               PSProteinInference & prot_inference,
                               Size & variable_counter,
                               libcpp_map[ String, libcpp_vector[ size_t ] ] & protein_feature_map,
                               Feature & new_feature, libcpp_map[ String, Size ] & protein_variable_index_map,
                               libcpp_map[ String, libcpp_set[ String ] ] & prot_id_counter) nogil except + # wrap-ignore

        void solveILP(libcpp_vector[ int ] & solution_indices) nogil except +

        void setLPSolver(SOLVER solver) nogil except +

        SOLVER getLPSolver() nogil except +

cdef extern from "<OpenMS/ANALYSIS/TARGETED/PSLPFormulation.h>" namespace "OpenMS::PSLPFormulation":
    
    cdef cppclass IndexTriple "OpenMS::PSLPFormulation::IndexTriple":
        IndexTriple() nogil except +
        IndexTriple(IndexTriple) nogil except + #wrap-ignore
        Size feature
        Int scan
        Size variable
        double rt_probability
        double signal_weight
        String prot_acc




import numpy as np
from scipy.signal import fftconvolve
import pylab as P
norm = lambda a : a / float(max(a))

N=4096

padded_N = N+9
i=np.arange(padded_N)
x= fftconvolve(np.random.uniform(0.5,1.0,N+9) * ( np.exp( -((i-400)/2500.0)**2 ) + 0.3*np.exp( -((i-padded_N/2.0)/100.0)**2 ) + 0.7*np.exp( -((i-padded_N)/400.0)**2 ) ), [1,2,3,4,4,3,2,1])[8:-8]
y= fftconvolve(np.random.uniform(0.5,1.0,padded_N) * ( 3.0*np.exp( -((i**0.9)/1000.0)**2 ) + np.exp( -((i-padded_N)/600.0)**2 ) ), [1,2,3,4,4,3,2,1])[8:-8]
x = norm(x)
y = norm(y)

outfile=open('x_and_y.txt', 'w')
outfile.write(str(N) + '\n')

for v in x:
  outfile.write(str(v) + ' ')
outfile.write('\n')
for v in y:
  outfile.write(str(v) + ' ')
outfile.write('\n')




from SourceFile cimport *
from DateTime cimport *
from DocumentIdentifier cimport *
from libcpp.vector cimport vector as libcpp_vector
from MetaInfoInterface cimport *

cdef extern from "<OpenMS/METADATA/ContactPerson.h>" namespace "OpenMS":

    cdef cppclass ContactPerson(MetaInfoInterface):
        # wrap-inherits:
        #    MetaInfoInterface

        ContactPerson() nogil except +
        ContactPerson(ContactPerson) nogil except + # wrap-ignore

        # returns the first name of the person
        String getFirstName() nogil except +
        # sets the first name of the person
        void setFirstName(String name) nogil except +

        # returns the last name of the person
        String getLastName() nogil except +
        # sets the last name of the person
        void setLastName(String name) nogil except +

        # sets the full name of the person (gets split into first and last name internally)
        void setName(String name) nogil except +

        # returns the affiliation
        String getInstitution() nogil except +
        # sets the affiliation
        void setInstitution(String institution) nogil except +

        # returns the email address
        String getEmail() nogil except +
        # sets the email address
        void setEmail(String email) nogil except +

        # returns the email address
        String getURL() nogil except +
        # sets the email address
        void setURL(String email) nogil except +

        # returns the address
        String getAddress() nogil except +
        # sets the address
        void setAddress(String email) nogil except +

        # returns miscellaneous info about the contact person
        String getContactInfo() nogil except +
        # sets miscellaneous info about the contact person
        void setContactInfo(String contact_info) nogil except +





from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from ConsensusMap cimport *
from ProgressLogger cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/BaseGroupFinder.h>" namespace "OpenMS":
    
    cdef cppclass BaseGroupFinder(DefaultParamHandler,ProgressLogger) :
        # wrap-ignore
        # no-pxd-import
        # ABSTRACT class
        # wrap-inherits:
        #  DefaultParamHandler
        #  ProgressLogger
        BaseGroupFinder() nogil except +
        BaseGroupFinder(BaseGroupFinder) nogil except + #wrap-ignore
        # void run(libcpp_vector[ ConsensusMap ] & input_, ConsensusMap & result) nogil except +
        void registerChildren() nogil except +




from Types cimport *

from MSExperiment cimport *
from MSSpectrum cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *
from AASequence cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/CHEMISTRY/SvmTheoreticalSpectrumGeneratorTrainer.h>" namespace "OpenMS":

    cdef cppclass SvmTheoreticalSpectrumGeneratorTrainer(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        SvmTheoreticalSpectrumGeneratorTrainer() nogil except +
        SvmTheoreticalSpectrumGeneratorTrainer(SvmTheoreticalSpectrumGeneratorTrainer) nogil except +

        void trainModel(MSExperiment & spectra, libcpp_vector[AASequence] & annotations, String filename, int precursor_charge) nogil except +
        void normalizeIntensity(MSSpectrum & S) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool

from String cimport *
from ProgressLogger cimport *

from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from PeakFileOptions cimport *

cdef extern from "<OpenMS/FORMAT/DTA2DFile.h>" namespace "OpenMS":

    cdef cppclass DTA2DFile(ProgressLogger):
        # wrap-inherits:
        #    ProgressLogger

        DTA2DFile() nogil except +
        DTA2DFile(DTA2DFile) nogil except +

        void storeTIC(String filename, MSExperiment & peakmap) nogil except +
        void store(String filename, MSExperiment & peakmap) nogil except +
        void load(String filename, MSExperiment & peakmap) nogil except +
        PeakFileOptions  getOptions() nogil except +




from SourceFile cimport *
from ContactPerson cimport *
from DateTime cimport *
from Sample cimport *
from DocumentIdentifier cimport *
from Instrument cimport *
from IonDetector cimport *
from HPLC cimport *
from ProteinIdentification cimport *
from MetaInfoInterface cimport *
from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/METADATA/ExperimentalSettings.h>" namespace "OpenMS":

    cdef cppclass ExperimentalSettings(MetaInfoInterface, DocumentIdentifier):
        # wrap-inherits:
        #    DocumentIdentifier
        #    MetaInfoInterface
        #
        # wrap-doc:
        #   Description of the experimental settings, provides meta-information
        #   about an LC-MS/MS injection.

        ExperimentalSettings() nogil except +
        ExperimentalSettings(ExperimentalSettings) nogil except + # wrap-ignore

        # returns a mutable reference to the source data file
        libcpp_vector[SourceFile] getSourceFiles() nogil except +
        # sets the source data file
        void setSourceFiles(libcpp_vector[SourceFile] source_files) nogil except +

        # returns the date the experiment was performed
        DateTime getDateTime() nogil except +
        # sets the date the experiment was performed
        void setDateTime(DateTime date_time) nogil except +

        # returns a mutable reference to the sample description
        Sample getSample() nogil except +
        # sets the sample description
        void setSample(Sample sample) nogil except +

        # returns a mutable reference to the list of contact persons
        libcpp_vector[ContactPerson] getContacts() nogil except +
        # sets the list of contact persons
        void setContacts(libcpp_vector[ContactPerson] contacts) nogil except +

        # returns a mutable reference to the MS instrument description
        Instrument getInstrument() nogil except +
        # sets the MS instrument description
        void setInstrument(Instrument instrument) nogil except +

        # returns a mutable reference to the description of the HPLC run
        HPLC getHPLC() nogil except +
        # sets the description of the HPLC run
        void setHPLC(HPLC hplc) nogil except +

        # returns the free-text comment
        String getComment() nogil except +
        # sets the free-text comment
        void setComment(String comment) nogil except +

        # returns a mutable reference to the protein ProteinIdentification vector
        libcpp_vector[ProteinIdentification] getProteinIdentifications() nogil except +
        # sets the protein ProteinIdentification vector
        void setProteinIdentifications(libcpp_vector[ProteinIdentification] protein_identifications) nogil except +

        # returns fraction identifier
        String getFractionIdentifier() nogil except +
        # sets the fraction identifier
        void setFractionIdentifier(String fraction_identifier) nogil except +




from Types cimport *
from String cimport *
from EmpiricalFormula cimport *
from ResidueModification cimport *

cdef extern from "<OpenMS/CHEMISTRY/Residue.h>" namespace "OpenMS":

    cdef cppclass Residue:
        # wrap-hash:
        #   getName().c_str()

        Residue() nogil except +
        Residue(Residue) nogil except + # wrap-ignore

        # detailed constructor
        Residue(String name,
                String three_letter_code,
                String one_letter_code,
                EmpiricalFormula formula) nogil except +

        # Conversions
        EmpiricalFormula getInternalToFull() nogil except +
        EmpiricalFormula getInternalToNTerm() nogil except +
        EmpiricalFormula getInternalToCTerm() nogil except +
        EmpiricalFormula getInternalToAIon() nogil except +
        EmpiricalFormula getInternalToBIon() nogil except +
        EmpiricalFormula getInternalToCIon() nogil except +
        EmpiricalFormula getInternalToXIon() nogil except +
        EmpiricalFormula getInternalToYIon() nogil except +
        EmpiricalFormula getInternalToZIon() nogil except +

        # returns the ion name given as a residue type
        String getResidueTypeName(ResidueType res_type) nogil except +

        # sets the name of the residue
        void setName(String name) nogil except +

        # returns the name of the residue
        String getName() nogil except +

        # sets the short name of the residue, this name is used in the PeptideSequence for output
        void setShortName(String short_name) nogil except +

        # returns the short name of the residue
        String getShortName() nogil except +

        # sets the synonyms
        void setSynonyms(libcpp_set[String] synonyms) nogil except +

        # adds a synonym
        void addSynonym(String synonym) nogil except +

        # returns the sysnonyms
        libcpp_set[String] getSynonyms() nogil except +

        # sets the name of the residue as three letter code
        void setThreeLetterCode(String three_letter_code) nogil except +

        # returns the name of the residue as three letter code
        String getThreeLetterCode() nogil except +

        # sets the name as one letter code
        void setOneLetterCode(String one_letter_code) nogil except +

        # returns the name as one letter code
        String getOneLetterCode() nogil except +

        # adds a neutral loss formula
        void addLossFormula(EmpiricalFormula) nogil except +

        # sets the neutral loss formulas
        void setLossFormulas(libcpp_vector[EmpiricalFormula]) nogil except +

        # adds N-terminal losses
        void addNTermLossFormula(EmpiricalFormula) nogil except +

        # sets the N-terminal losses
        void setNTermLossFormulas(libcpp_vector[EmpiricalFormula]) nogil except +

        # returns the neutral loss formulas
        libcpp_vector[EmpiricalFormula] getLossFormulas() nogil except +

        # returns N-terminal loss formulas
        libcpp_vector[EmpiricalFormula] getNTermLossFormulas() nogil except +

        # set the neutral loss molecule name
        void setLossNames(libcpp_vector[String] name) nogil except +

        # sets the N-terminal loss names
        void setNTermLossNames(libcpp_vector[String] name) nogil except +

        # add neutral loss molecule name
        void addLossName(String name) nogil except +

        # adds a N-terminal loss name
        void addNTermLossName(String name) nogil except +

        # gets neutral loss name (if there is one, else returns an empty string)
        libcpp_vector[String] getLossNames() nogil except +

        # returns the N-terminal loss names
        libcpp_vector[String] getNTermLossNames() nogil except +

        # set empirical formula of the residue (must be full, with N and C-terminus)
        void setFormula(EmpiricalFormula formula) nogil except +

        # returns the empirical formula of the residue
        EmpiricalFormula getFormula() nogil except +
        EmpiricalFormula getFormula(ResidueType res_type) nogil except +

        # sets average weight of the residue (must be full, with N and C-terminus)
        void setAverageWeight(double weight) nogil except +

        # returns average weight of the residue
        double getAverageWeight() nogil except +
        double getAverageWeight(ResidueType res_type) nogil except +

        # sets monoisotopic weight of the residue (must be full, with N and C-terminus)
        void setMonoWeight(double weight) nogil except +

        # returns monoisotopic weight of the residue
        double getMonoWeight() nogil except +
        double getMonoWeight(ResidueType res_type) nogil except +

        const ResidueModification * getModification() nogil except +

        # sets the modification by name; the mod should be present in ModificationsDB
        void setModification(String name) nogil except +

        # returns the name of the modification to the modification
        String getModificationName() nogil except +

        # sets the low mass marker ions as a vector of formulas
        void setLowMassIons(libcpp_vector[EmpiricalFormula] low_mass_ions) nogil except +

        # returns a vector of formulas with the low mass markers of the residue
        libcpp_vector[EmpiricalFormula] getLowMassIons() nogil except +

        # sets the residue sets the amino acid is contained in
        void setResidueSets(libcpp_set[String] residues_sets) nogil except +

        # adds a residue set to the residue sets
        void addResidueSet(String residue_sets) nogil except +

        # returns the residue sets this residue is contained in
        libcpp_set[String] getResidueSets() nogil except +

        # true if the residue has neutral loss
        bool hasNeutralLoss() nogil except +

        # true if N-terminal neutral losses are set
        bool hasNTermNeutralLosses() nogil except +

        # equality operator
        bool operator==(Residue & residue) nogil except +

        # inequality operator
        bool operator!=(Residue & residue) nogil except +

        # equality operator for one letter code
        bool operator==(char one_letter_code) nogil except +

        # equality operator for one letter code
        bool operator!=(char one_letter_code) nogil except +

        # returns the pka of the residue
        double getPka() nogil except +

        # returns the pkb of the residue
        double getPkb() nogil except +

        # returns the pkc of the residue if it exists otherwise -1
        double getPkc() nogil except +

        # calculates the isoelectric point using the pk* values
        double getPiValue() nogil except +

        # sets the pka of the residue
        void setPka(double value) nogil except +

        # sets the pkb of the residue
        void setPkb(double value) nogil except +

        # sets the pkc of the residue
        void setPkc(double value) nogil except +

        # returns the side chain basicity
        double getSideChainBasicity() nogil except +

        # sets the side chain basicity
        void setSideChainBasicity(double gb_sc) nogil except +

        # returns the backbone basicitiy if located in N-terminal direction
        double getBackboneBasicityLeft() nogil except +

        # sets the N-terminal direction backbone basicitiy
        void setBackboneBasicityLeft(double gb_bb_l) nogil except +

        # returns the C-terminal direction backbone basicitiy
        double getBackboneBasicityRight() nogil except +

        # sets the C-terminal direction backbone basicity
        void setBackboneBasicityRight(double gb_bb_r) nogil except +

        # true if the residue is a modified one
        bool isModified() nogil except +

        # true if the residue is contained in the set
        bool isInResidueSet(String residue_set) nogil except +

        # helper for mapping residue types to letters for Text annotations and labels
        char residueTypeToIonLetter(ResidueType res_type) nogil except +

cdef extern from "<OpenMS/CHEMISTRY/Residue.h>" namespace "OpenMS::Residue":

    cdef enum ResidueType:
      # wrap-attach:
      #   Residue
      Full = 0,       # with N-terminus and C-terminus
      Internal,       # internal, without any termini
      NTerminal,      # only N-terminus
      CTerminal,      # only C-terminus
      AIon,           # MS:1001229 N-terminus up to the C-alpha/carbonyl carbon bond
      BIon,           # MS:1001224 N-terminus up to the peptide bond
      CIon,           # MS:1001231 N-terminus up to the amide/C-alpha bond
      XIon,           # MS:1001228 amide/C-alpha bond up to the C-terminus
      YIon,           # MS:1001220 peptide bond up to the C-terminus
      ZIon,           # MS:1001230 C-alpha/carbonyl carbon bond
      Precursor_ion,  # MS:1001523 Precursor ion
      BIonMinusH20,   # MS:1001222 b ion without water
      YIonMinusH20,   # MS:1001223 y ion without water
      BIonMinusNH3,   # MS:1001232 b ion without ammonia
      YIonMinusNH3,   # MS:1001233 y ion without ammonia
      NonIdentified,  # MS:1001240 Non-identified ion
      Unannotated,    # no stored annotation
      SizeOfResidueType



from libcpp.pair cimport pair as libcpp_pair
from Types cimport *
from String cimport *
from StringList cimport *
from ConvexHull2D cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/FeatureFinderAlgorithmPickedHelperStructs.h>" namespace "OpenMS::FeatureFinderAlgorithmPickedHelperStructs":

    cdef cppclass TheoreticalIsotopePattern "OpenMS::FeatureFinderAlgorithmPickedHelperStructs::TheoreticalIsotopePattern":
        TheoreticalIsotopePattern() nogil except + # wrap-ignore
        TheoreticalIsotopePattern(TheoreticalIsotopePattern) nogil except + #wrap-ignore

        libcpp_vector[ double ] intensity
        Size optional_begin
        Size optional_end
        double max
        Size trimmed_left

        Size size() nogil except +

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/FeatureFinderAlgorithmPickedHelperStructs.h>" namespace "OpenMS::FeatureFinderAlgorithmPickedHelperStructs":

    # Since this is a templated class, we cannot tell Cython what the C++
    # equivalent would be and we need to name it MassTrace
    cdef cppclass MassTrace:
        MassTrace(MassTrace) nogil except + #wrap-ignore

        # POINTER # PeakType * max_peak
        double max_rt
        double theoretical_int
        # POINTER # libcpp_vector[ libcpp_pair[ double, PeakType * ] ] peaks
        ConvexHull2D getConvexhull() nogil except +
        void updateMaximum() nogil except +
        double getAvgMZ() nogil except +
        bool isValid() nogil except +

    # Since this is a templated class, we cannot tell Cython what the C++
    # equivalent would be and we need to name it MassTraces
    cdef cppclass MassTraces:
        MassTraces() nogil except +
        MassTraces(MassTraces) nogil except + #wrap-ignore

        Size max_trace
        double baseline
        Size getPeakCount() nogil except +
        bool isValid(double seed_mz, double trace_tolerance) nogil except +
        Size getTheoreticalmaxPosition() nogil except +
        void updateBaseline() nogil except +
        libcpp_pair[ double, double ] getRTBounds() nogil except +

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/FeatureFinderAlgorithmPickedHelperStructs.h>" namespace "OpenMS::FeatureFinderAlgorithmPickedHelperStructs":

    cdef cppclass Seed "OpenMS::FeatureFinderAlgorithmPickedHelperStructs::Seed":
        Seed(Seed) nogil except + #wrap-ignore
        Size spectrum
        Size peak
        float intensity
        bool operator<(Seed & rhs) nogil except +

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/FeatureFinderAlgorithmPickedHelperStructs.h>" namespace "OpenMS::FeatureFinderAlgorithmPickedHelperStructs":

    cdef cppclass IsotopePattern "OpenMS::FeatureFinderAlgorithmPickedHelperStructs::IsotopePattern":
        IsotopePattern(IsotopePattern) nogil except + #wrap-ignore

        # TODO STL attributes -- Signed size does not work either!
        # vector.from_py:33:13: 'ptrdiff_t' is not a type identifier
        # libcpp_vector[ SignedSize ] peak
        libcpp_vector[ size_t ] spectrum
        libcpp_vector[ double ] intensity
        libcpp_vector[ double ] mz_score
        libcpp_vector[ double ] theoretical_mz
        TheoreticalIsotopePattern theoretical_pattern
        IsotopePattern(Size size) nogil except +





from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Residue cimport *
from DefaultParamHandler cimport *
from AASequence cimport *
from MSSpectrum cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/CHEMISTRY/TheoreticalSpectrumGenerator.h>" namespace "OpenMS":
    
    cdef cppclass TheoreticalSpectrumGenerator(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        TheoreticalSpectrumGenerator() nogil except +
        TheoreticalSpectrumGenerator(TheoreticalSpectrumGenerator) nogil except +
        void getSpectrum(MSSpectrum &spec, AASequence &peptide, Int min_charge, Int max_charge) nogil except +



from Types cimport *
from SwathMap cimport *
from String cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/SwathWindowLoader.h>" namespace "OpenMS":

    cdef cppclass SwathWindowLoader:

        SwathWindowLoader() nogil except +
        SwathWindowLoader(SwathWindowLoader) nogil except +

        void annotateSwathMapsFromFile(String filename,
                                       libcpp_vector[ SwathMap ]& swath_maps, bool do_sort, bool force) nogil except +

        void readSwathWindows(String filename,
                              libcpp_vector[double]& swath_prec_lower,
                              libcpp_vector[double]& swath_prec_upper) nogil except +




from libcpp.string cimport string as libcpp_string
from libcpp.vector cimport vector as libcpp_vector
from String cimport *
from StringList cimport *
from IntList cimport *
from DoubleList cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/DataValue.h>" namespace "OpenMS":

    cdef cppclass DataValue:
         DataValue() nogil except +
         DataValue(DataValue) nogil except + # wrap-ignore
         DataValue(char *) nogil except +
         DataValue(const String&) nogil except +
         DataValue(int) nogil except +
         DataValue(double) nogil except +
         DataValue(StringList) nogil except +
         DataValue(IntList) nogil except +
         DataValue(DoubleList) nogil except +

         #conversion ops, different declarations as in c++ !
         int operator()(int) nogil except + #wrap-cast:toInt
         String operator()(DataValue) nogil except + #wrap-cast:toString
         double operator()(DataValue) nogil except + #wrap-cast:toDouble
         StringList toStringList() nogil except +
         libcpp_vector[ double ] toDoubleList() nogil except +
         libcpp_vector[ int ] toIntList() nogil except +
         String toString() nogil except +
         bool toBool() nogil except +

         DataType valueType() nogil except +

         int isEmpty() nogil except +

         UnitType getUnitType() nogil except +
         void setUnitType(UnitType u) nogil except +

         bool hasUnit() nogil except +
         int getUnit() nogil except +
         void setUnit(int unit_id) nogil except +

cdef extern from "<OpenMS/DATASTRUCTURES/DataValue.h>" namespace "OpenMS::DataValue":

    cdef enum DataType "OpenMS::DataValue::DataType":
        STRING_VALUE # string value
        INT_VALUE # integer value
        DOUBLE_VALUE # double value
        STRING_LIST # string list
        INT_LIST # integer list
        DOUBLE_LIST # double list
        EMPTY_VALUE # empty value

    cdef enum UnitType "OpenMS::DataValue::UnitType":
        UNIT_ONTOLOGY # unit.ontology UO:
        MS_ONTOLOGY # ms.ontology MS:
        OTHER # undefined ontology




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from AASequence cimport *
from LocalLinearMap cimport *

cdef extern from "<OpenMS/ANALYSIS/PIP/PeakIntensityPredictor.h>" namespace "OpenMS":
    
    cdef cppclass PeakIntensityPredictor "OpenMS::PeakIntensityPredictor":
        PeakIntensityPredictor() nogil except +
        PeakIntensityPredictor(PeakIntensityPredictor) nogil except + #wrap-ignore
        double predict(AASequence & sequence) nogil except +
        double predict(AASequence & sequence, libcpp_vector[ double ] & add_info) nogil except +
        libcpp_vector[ double ] predict(libcpp_vector[ AASequence ] & sequences) nogil except +
        libcpp_vector[ double ] predict(libcpp_vector[ AASequence ] & sequences, libcpp_vector[ libcpp_vector[ double ] ] & add_info) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool

from String cimport *
from ProgressLogger cimport *

from MSSpectrum cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/FORMAT/DTAFile.h>" namespace "OpenMS":

    cdef cppclass DTAFile:

        DTAFile() nogil except +
        DTAFile(DTAFile) nogil except +

        void load(String filename, MSSpectrum & spectrum) nogil except +
        void store(String filename, MSSpectrum & spectrum) nogil except +



from Types cimport *
from OpenSwathDataStructures cimport *
from SpectrumAccessTransforming cimport *
from SpectrumAccessOpenMS cimport *
from SpectrumAccessOpenMSCached cimport *
from SpectrumAccessOpenMSInMemory cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/DATAACCESS/SpectrumAccessQuadMZTransforming.h>" namespace "OpenMS":
    
    cdef cppclass SpectrumAccessQuadMZTransforming(SpectrumAccessTransforming) :
        # wrap-inherits:
        #  SpectrumAccessTransforming

        SpectrumAccessQuadMZTransforming() nogil except + # wrap-pass-constructor
        SpectrumAccessQuadMZTransforming(SpectrumAccessQuadMZTransforming) nogil except + #wrap-ignore

        # SpectrumAccessQuadMZTransforming(shared_ptr[ ISpectrumAccess] sptr, double a, double b, double c, bool ppm) nogil except +
        SpectrumAccessQuadMZTransforming(shared_ptr[ SpectrumAccessOpenMS ], double a, double b, double c, bool ppm) nogil except +
        SpectrumAccessQuadMZTransforming(shared_ptr[ SpectrumAccessOpenMSCached ], double a, double b, double c, bool ppm) nogil except +
        SpectrumAccessQuadMZTransforming(shared_ptr[ SpectrumAccessOpenMSInMemory ], double a, double b, double c, bool ppm) nogil except +




from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from String cimport *
from PeptideIdentification cimport *
from ProteinIdentification cimport *
from DateTime cimport *

cdef extern from "<OpenMS/FORMAT/SequestOutfile.h>" namespace "OpenMS":
    
    cdef cppclass SequestOutfile "OpenMS::SequestOutfile":
        SequestOutfile() nogil except +
        SequestOutfile(SequestOutfile) nogil except +

        bool operator==(SequestOutfile &sequest_outfile) nogil except +
        void load(const String &result_filename,
                  libcpp_vector[ PeptideIdentification ] &peptide_identifications,
                  ProteinIdentification &protein_identification,
                  double p_value_threshold,
                  libcpp_vector[ double ] &pvalues,
                  const String &database,
                  bool ignore_proteins_per_peptide) nogil except +
        bool getColumns(const String &line, libcpp_vector[ String ] &substrings, Size number_of_columns, Size reference_column) nogil except +
        void getSequences(const String &database_filename,
                          libcpp_map[ String, size_t ] &ac_position_map,
                          libcpp_vector[ String ] &sequences,
                          libcpp_vector[ libcpp_pair[ String, size_t ] ] &found,
                          libcpp_map[ String, size_t ] &not_found) nogil except + # wrap-ignore
        void getACAndACType(String line, String &accession, String &accession_type) nogil except +

        # TODO immutable types by reference
        # 
        # void readOutHeader(String &result_filename, DateTime &datetime,
        # double &precursor_mz_value, Int &charge, Size &precursor_mass_type,
        # Size &ion_mass_type, Size &displayed_peptides, String &sequest,
        # String &sequest_version, String &database_type, Int &number_column,
        # Int &rank_sp_column, Int &id_column, Int &mh_column, Int
        # &delta_cn_column, Int &xcorr_column, Int &sp_column, Int &sf_column,
        # Int &ions_column, Int &reference_column, Int &peptide_column, Int
        # &score_column, Size &number_of_columns) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool

from DefaultParamHandler cimport *
from PeptideIdentification cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/IDDecoyProbability.h>" namespace "OpenMS":

    cdef cppclass IDDecoyProbability(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        IDDecoyProbability()                    nogil except +
        IDDecoyProbability(IDDecoyProbability)  nogil except +

        void apply(libcpp_vector[PeptideIdentification] & prob_ids, libcpp_vector[PeptideIdentification] & fwd_ids, libcpp_vector[PeptideIdentification] & rev_ids)  nogil except +
        void apply(libcpp_vector[PeptideIdentification] & ids)  nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.string cimport string as libcpp_string
from libcpp.vector cimport vector as libcpp_vector
from SwathMap cimport *
from MRMFeatureFinderScoring cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/SwathMapMassCorrection.h>" namespace "OpenMS":
    
    cdef cppclass SwathMapMassCorrection "OpenMS::SwathMapMassCorrection":
        SwathMapMassCorrection(SwathMapMassCorrection) nogil except + #wrap-ignore
        # NAMESPACE # void correctMZ(const OpenMS::MRMFeatureFinderScoring::TransitionGroupMapType & transition_group_map, libcpp_vector[ OpenSwath::SwathMap ] & swath_maps, const libcpp_string & corr_type, const double mz_extr_window, const bool ppm) nogil except +






    def getSpectrumAlignment(self, list result, spec1, spec2):
        # TODO: use autowrap for this directly
        assert isinstance(spec1, (MSSpectrum))
        assert isinstance(spec2, (MSSpectrum))
        cdef _MSSpectrum * _new_spec1 = new _MSSpectrum()
        cdef _MSSpectrum * _new_spec2 = new _MSSpectrum()
        cdef _Peak1D _peak

        if True:
            for _peak in deref(<_MSSpectrum *>(<MSSpectrum>spec1).inst.get()):
                _new_spec1.push_back(<_Peak1D>_peak)

        if True:
            for _peak in deref(<_MSSpectrum *>(<MSSpectrum>spec2).inst.get()):
                _new_spec2.push_back(<_Peak1D>_peak)

        cdef libcpp_vector[libcpp_pair[Size, Size]] _result

        self.inst.get().getSpectrumAlignment(_result, deref(_new_spec1), deref(_new_spec2))

        result[:] = []

        cdef libcpp_vector[libcpp_pair[Size, Size]].iterator _it =  _result.begin()
        while _it != _result.end():
            result.append((<int>(deref(_it).first), <int>(deref(_it).second)))
            inc(_it)

        del _new_spec1
        del _new_spec2






# from BaseModel cimport *
from LinearInterpolation cimport *
from DPosition cimport *

ctypedef double IntensityType
ctypedef DPosition1 PositionType
ctypedef double CoordinateType
# ctypedef Math::LinearInterpolation<double> LinearInterpolation

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/InterpolationModel.h>" namespace "OpenMS":
    
    cdef cppclass InterpolationModel "OpenMS::InterpolationModel":
        InterpolationModel() nogil except +
        InterpolationModel(InterpolationModel) nogil except +
        
        # double getIntensity(DPosition1 &pos) nogil except +
        double getIntensity(double coord) nogil except +
        double getScalingFactor() nogil except +
        void setOffset(double offset) nogil except +
        double getCenter() nogil except +
        void setSamples() nogil except +
        void setInterpolationStep(double interpolation_step) nogil except +
        void setScalingFactor(double scaling) nogil except +

        LinearInterpolation[double,double] getInterpolation() nogil except +
        # void getSamples(SamplesType &cont) nogil except +
        # typedef typename DPeak<D>::Type PeakType;
        # typedef std::vector<PeakType> SamplesType;




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from String cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/LPWrapper.h>" namespace "OpenMS":
    
    cdef cppclass LPWrapper "OpenMS::LPWrapper":
        LPWrapper() nogil except +
        LPWrapper(LPWrapper) nogil except + #wrap-ignore
        Int addRow(libcpp_vector[ int ] row_indices, libcpp_vector[ double ] row_values, const String & name) nogil except +
        Int addColumn() nogil except +
        Int addColumn(libcpp_vector[ int ] column_indices, libcpp_vector[ double ] column_values, const String & name) nogil except +
        Int addRow(libcpp_vector[ int ] & row_indices, libcpp_vector[ double ] & row_values, const String & name, double lower_bound, double upper_bound, LPWrapper_Type type_) nogil except +
        Int addColumn(libcpp_vector[ int ] & column_indices, libcpp_vector[ double ] & column_values, const String & name, double lower_bound, double upper_bound, LPWrapper_Type type_) nogil except +
        void deleteRow(Int index) nogil except +
        void setColumnName(Int index, const String & name) nogil except +
        String getColumnName(Int index) nogil except +
        String getRowName(Int index) nogil except +
        Int getRowIndex(const String & name) nogil except +
        Int getColumnIndex(const String & name) nogil except +
        double getColumnUpperBound(Int index) nogil except +
        double getColumnLowerBound(Int index) nogil except +
        double getRowUpperBound(Int index) nogil except +
        double getRowLowerBound(Int index) nogil except +
        void setRowName(Int index, const String & name) nogil except +
        void setColumnBounds(Int index, double lower_bound, double upper_bound, LPWrapper_Type type_) nogil except +
        void setRowBounds(Int index, double lower_bound, double upper_bound, LPWrapper_Type type_) nogil except +
        void setColumnType(Int index, VariableType type_) nogil except +
        VariableType getColumnType(Int index) nogil except +
        void setObjective(Int index, double obj_value) nogil except +
        double getObjective(Int index) nogil except +
        void setObjectiveSense(Sense sense) nogil except +
        Sense getObjectiveSense() nogil except +
        Int getNumberOfColumns() nogil except +
        Int getNumberOfRows() nogil except +
        void setElement(Int row_index, Int column_index, double value) nogil except +
        double getElement(Int row_index, Int column_index) nogil except +
        void readProblem(String filename, String format_) nogil except +
        void writeProblem(const String & filename, WriteFormat format_) nogil except +
        Int solve(SolverParam & solver_param, Size verbose_level) nogil except +
        SolverStatus getStatus() nogil except +
        double getObjectiveValue() nogil except +
        double getColumnValue(Int index) nogil except +
        Int getNumberOfNonZeroEntriesInRow(Int idx) nogil except +
        void getMatrixRow(Int idx, libcpp_vector[ int ] & indexes) nogil except +
        SOLVER getSolver() nogil except +

cdef extern from "<OpenMS/DATASTRUCTURES/LPWrapper.h>" namespace "OpenMS::LPWrapper":
    
    cdef cppclass SolverParam "OpenMS::LPWrapper::SolverParam":
        SolverParam() nogil except +
        SolverParam(SolverParam) nogil except + #wrap-ignore
        Int message_level
        Int branching_tech
        Int backtrack_tech
        Int preprocessing_tech
        bool enable_feas_pump_heuristic
        bool enable_gmi_cuts
        bool enable_mir_cuts
        bool enable_cov_cuts
        bool enable_clq_cuts
        double mip_gap
        Int time_limit
        Int output_freq
        Int output_delay
        bool enable_presolve
        bool enable_binarization


cdef extern from "<OpenMS/DATASTRUCTURES/LPWrapper.h>" namespace "OpenMS::LPWrapper":
    cdef enum LPWrapper_Type "OpenMS::LPWrapper::Type":
        #wrap-attach:
        #    LPWrapper
        UNBOUNDED
        LOWER_BOUND_ONLY
        UPPER_BOUND_ONLY
        DOUBLE_BOUNDED
        FIXED

cdef extern from "<OpenMS/DATASTRUCTURES/LPWrapper.h>" namespace "OpenMS::LPWrapper":
    cdef enum VariableType "OpenMS::LPWrapper::VariableType":
        #wrap-attach:
        #    LPWrapper
        CONTINUOUS
        INTEGER
        BINARY

cdef extern from "<OpenMS/DATASTRUCTURES/LPWrapper.h>" namespace "OpenMS::LPWrapper":
    cdef enum Sense "OpenMS::LPWrapper::Sense":
        #wrap-attach:
        #    LPWrapper
        MIN
        MAX

cdef extern from "<OpenMS/DATASTRUCTURES/LPWrapper.h>" namespace "OpenMS::LPWrapper":
    cdef enum WriteFormat "OpenMS::LPWrapper::WriteFormat":
        #wrap-attach:
        #    LPWrapper
        FORMAT_LP
        FORMAT_MPS
        FORMAT_GLPK

cdef extern from "<OpenMS/DATASTRUCTURES/LPWrapper.h>" namespace "OpenMS::LPWrapper":
    cdef enum SOLVER "OpenMS::LPWrapper::SOLVER":
        #wrap-attach:
        #    LPWrapper
        SOLVER_GLPK

cdef extern from "<OpenMS/DATASTRUCTURES/LPWrapper.h>" namespace "OpenMS::LPWrapper":
    cdef enum SolverStatus "OpenMS::LPWrapper::SolverStatus":
        #wrap-attach:
        #    LPWrapper
        UNDEFINED
        OPTIMAL
        FEASIBLE
        NO_FEASIBLE_SOL




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from Peak1D cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/ContinuousWaveletTransform.h>" namespace "OpenMS":
    
    cdef cppclass ContinuousWaveletTransform "OpenMS::ContinuousWaveletTransform":
        ContinuousWaveletTransform() nogil except +
        ContinuousWaveletTransform(ContinuousWaveletTransform) nogil except + #wrap-ignore
        libcpp_vector[ Peak1D ]  getSignal() nogil except +
        void setSignal(libcpp_vector[ Peak1D ] & signal) nogil except +
        libcpp_vector[ double ]  getWavelet() nogil except +
        void setWavelet(libcpp_vector[ double ] & wavelet) nogil except +
        double  getScale() nogil except +
        void setScale(double scale) nogil except +
        double  getSpacing() nogil except +
        void setSpacing(double spacing) nogil except +
        SignedSize getLeftPaddingIndex() nogil except +
        void setLeftPaddingIndex(SignedSize end_left_padding) nogil except +
        SignedSize getRightPaddingIndex() nogil except +
        void setRightPaddingIndex(SignedSize begin_right_padding) nogil except +
        SignedSize getSignalLength() nogil except +
        void setSignalLength(SignedSize signal_length) nogil except +
        int getSize() nogil except +
        void init(double scale, double spacing) nogil except +
        # double <](unsigned int i) nogil except +
        # double <](unsigned int i) nogil except +




from Types cimport *
from OPXLDataStructs cimport *

cdef extern from "<OpenMS/ANALYSIS/XLMS/OPXLDataStructs.h>" namespace "OpenMS":

    cdef cppclass XLPrecursor "OpenMS::OPXLDataStructs::XLPrecursor":

        XLPrecursor(XLPrecursor) nogil except +
        XLPrecursor() nogil except +

        float precursor_mass
        unsigned int alpha_index
        unsigned int beta_index
